#!BPY
# coding: utf-8
"""
 Name: 'MikuMikuDance model (.pmd)...'
 Blender: 248
 Group: 'Export'
 Tooltip: 'Export PMD file for MikuMikuDance.'
"""
__author__= ["ousttrue"]
__version__= "2.5"
__url__=()
__bpydoc__="""
pmd Importer

This script exports a pmd model.

20100318: first implementation.
20100519: refactoring. use C extension.
20100530: implement basic features.
20100612: integrate 2.4 and 2.5.
20100616: implement rigid body.
20100619: fix rigid body, bone weight.
20100626: refactoring.
20100629: sphere map.
20100710: toon texture & bone group.
20100711: separate vertex with normal or uv.
20100724: update for Blender2.53.
20100731: add full python module.
20101005: update for Blender2.54.
20101228: update for Blender2.55.
20110429: update for Blender2.57b.
20110522: implement RigidBody and Constraint.
20111002: update for pymeshio-2.1.0
20120117: fix empty morph bug(ciritical).
"""
import io

from . import bl
from . import exporter
from .pymeshio import common
from .pymeshio import pmd
from .pymeshio import englishmap
from .pymeshio.pmd import writer


def near(x, y, EPSILON=1e-5):
    d=x-y
    return d>=-EPSILON and d<=EPSILON


def toCP932(s):
    return s.encode('cp932')


def write(self, path):
    model=pmd.Model()

    o=self.root.o
    englishName=o.name
    name=o[bl.MMD_MB_NAME] if bl.MMD_MB_NAME in o else 'Blenderエクスポート'
    comment=o[bl.MMD_MB_COMMENT] if bl.MMD_MB_COMMENT in o else 'Blnderエクスポート\n'
    englishComment=o[bl.MMD_COMMENT] if bl.MMD_COMMENT in o else 'blender export\n'

    model.name=name.encode('cp932')
    model.english_name=englishName.encode('cp932')
    model.comment=comment.encode('cp932')
    model.english_comment=englishComment.encode('cp932')

    # 頂点
    model.vertices=[pmd.Vertex(
        # convert right-handed z-up to left-handed y-up
        common.Vector3(pos[0], pos[2], pos[1]), 
        # convert right-handed z-up to left-handed y-up
        common.Vector3(attribute.nx, attribute.nz, attribute.ny),
        # reverse vertical
        common.Vector2(attribute.u, 1.0-attribute.v),
        self.skeleton.indexByName(b0),
        self.skeleton.indexByName(b1),
        int(100*weight),
        # edge flag, 0: enable edge, 1: not edge
        0 
        )
        for pos, attribute, b0, b1, weight in self.oneSkinMesh.vertexArray.zip()]

    # 面とマテリアル
    vertexCount=self.oneSkinMesh.getVertexCount()
    for material_name, indices in self.oneSkinMesh.vertexArray.each():
        #print('material:', material_name)
        try:
            m=bl.material.get(material_name)
        except KeyError as e:
            m=DefaultMatrial()
        def get_texture_name(texture):
            pos=texture.replace("\\", "/").rfind("/")
            if pos==-1:
                return texture
            else:
                return texture[pos+1:]
        textures=[get_texture_name(path)
            for path in bl.material.eachEnalbeTexturePath(m)]
        #print(textures)
        # マテリアル
        model.materials.append(pmd.Material(
                # diffuse_color
                common.RGB(m.diffuse_color[0], m.diffuse_color[1], m.diffuse_color[2]),
                m.alpha,
                # specular_factor
                0 if m.specular_toon_size<1e-5 else m.specular_hardness*10,
                # specular_color
                common.RGB(m.specular_color[0], m.specular_color[1], m.specular_color[2]),
                # ambient_color
                common.RGB(m.mirror_color[0], m.mirror_color[1], m.mirror_color[2]),
                # flag
                1 if m.subsurface_scattering.use else 0,
                # toon
                0,
                # vertex_count
                len(indices),
                # texture
                ('*'.join(textures) if len(textures)>0 else "").encode('cp932')
                ))
        # 面
        for i in indices:
            assert(i<vertexCount)
        for i in range(0, len(indices), 3):
            # reverse triangle
            model.indices.append(indices[i])
            model.indices.append(indices[i+1])
            model.indices.append(indices[i+2])

    # bones
    boneNameMap={}
    for i, b in enumerate(self.skeleton.bones):

        # name
        boneNameMap[b.name]=i
        v=englishmap.getUnicodeBoneName(b.name)
        if not v:
            v=[b.name, b.name]
        assert(v)
        bone=pmd.Bone(v[1].encode('cp932'))

        # english name
        bone_english_name=toCP932(b.name)
        if len(bone_english_name)>=20:
            print('bone_english_name', bone_english_name)
            #assert(len(bone_english_name)<20)
        bone.english_name=bone_english_name

        if len(v)>=3:
            # has type
            if v[2]==5:
                b.ik_index=self.skeleton.indexByName('eyes')
            bone.type=v[2]
        else:
            bone.type=b.type

        bone.parent_index=b.parent_index
        bone.tail_index=b.tail_index
        bone.ik_index=b.ik_index

        # convert right-handed z-up to left-handed y-up
        bone.pos.x=b.pos[0] if not near(b.pos[0], 0) else 0
        bone.pos.y=b.pos[2] if not near(b.pos[2], 0) else 0
        bone.pos.z=b.pos[1] if not near(b.pos[1], 0) else 0
        
        model.bones.append(bone)

    # IK
    for ik in self.skeleton.ik_list:
        solver=pmd.IK()
        solver.index=self.skeleton.getIndex(ik.target)
        solver.target=self.skeleton.getIndex(ik.effector)
        solver.length=ik.length
        b=self.skeleton.bones[ik.effector.parent_index]
        for i in range(solver.length):
            solver.children.append(self.skeleton.getIndex(b))
            b=self.skeleton.bones[b.parent_index]
        solver.iterations=ik.iterations
        solver.weight=ik.weight
        model.ik_list.append(solver)

    # 表情
    for i, m in enumerate(self.oneSkinMesh.morphList):
        v=englishmap.getUnicodeSkinName(m.name)
        if not v:
            v=[m.name, m.name, 0]
        assert(v)
        # morph
        morph=pmd.Morph(v[1].encode("cp932"))
        morph.english_name=m.name.encode("cp932")
        m.type=v[2]
        morph.type=v[2]
        for index, offset in m.offsets:
            # convert right-handed z-up to left-handed y-up
            morph.append(index, offset[0], offset[2], offset[1])
        morph.vertex_count=len(m.offsets)
        model.morphs.append(morph)

    # 表情枠
    # type==0はbase
    for i, m in enumerate(self.oneSkinMesh.morphList):
        if m.type==3:
            model.morph_indices.append(i)
    for i, m in enumerate(self.oneSkinMesh.morphList):
        if m.type==2:
            model.morph_indices.append(i)
    for i, m in enumerate(self.oneSkinMesh.morphList):
        if m.type==1:
            model.morph_indices.append(i)
    for i, m in enumerate(self.oneSkinMesh.morphList):
        if m.type==4:
            model.morph_indices.append(i)

    # ボーングループ
    for g in self.skeleton.bone_groups:
        name=englishmap.getUnicodeBoneGroupName(g[0])
        if not name:
            name=g[0]
        englishName=g[0]

        model.bone_group_list.append(pmd.BoneGroup(
                (name+'\n').encode('cp932'),
                (englishName+'\n').encode('cp932')
                ))

    # ボーングループメンバー
    for i, b in enumerate(self.skeleton.bones):
        if i==0:
           continue
        if b.type in [6, 7]:
           continue
        model.bone_display_list.append((i, self.skeleton.getBoneGroup(b)))

    # toon
    toonMeshObject=None
    for o in bl.object.each():
        try:
            if o.name.startswith(bl.TOON_TEXTURE_OBJECT):
                toonMeshObject=o
        except:
            p(o.name)
        break
    if toonMeshObject:
        toonMesh=bl.object.getData(toonMeshObject)
        toonMaterial=bl.mesh.getMaterial(toonMesh, 0)
        for i in range(10):
            t=bl.material.getTexture(toonMaterial, i)
            if t:
                model.toon_textures[i]=("%s" % t.name).encode('cp932')
            else:
                model.toon_textures[i]=("toon%02d.bmp" % (i+1)).encode('cp932')
    else:
        for i in range(10):
            model.toon_textures[i]=("toon%02d.bmp" % (i+1)).encode('cp932')

    # rigid body
    rigidNameMap={}
    for i, obj in enumerate(self.oneSkinMesh.rigidbodies):
        name=obj[bl.RIGID_NAME] if bl.RIGID_NAME in obj else obj.name
        #print('rigidbody', name)
        rigidNameMap[name]=i
        boneIndex=boneNameMap[obj[bl.RIGID_BONE_NAME]]
        if boneIndex==0:
            boneIndex=-1
            bone=self.skeleton.bones[0]
        else:
            bone=self.skeleton.bones[boneIndex]
        # x, z, y -> x, y, z
        if obj[bl.RIGID_SHAPE_TYPE]==0:
            shape_type=pmd.SHAPE_SPHERE
            shape_size=common.Vector3(obj.scale[0], 0, 0)
        elif obj[bl.RIGID_SHAPE_TYPE]==1:
            shape_type=pmd.SHAPE_BOX
            shape_size=common.Vector3(obj.scale[0], obj.scale[2], obj.scale[1])
        elif obj[bl.RIGID_SHAPE_TYPE]==2:
            shape_type=pmd.SHAPE_CAPSULE
            shape_size=common.Vector3(obj.scale[0], obj.scale[2], 0)
        rigidBody=pmd.RigidBody(
                name.encode('cp932'), 
                collision_group=obj[bl.RIGID_GROUP],
                no_collision_group=obj[bl.RIGID_INTERSECTION_GROUP],
                bone_index=boneIndex,
                shape_position=common.Vector3(
                    obj.location.x-bone.pos[0],
                    obj.location.z-bone.pos[2],
                    obj.location.y-bone.pos[1]),
                shape_rotation=common.Vector3(
                    -obj.rotation_euler[0],
                    -obj.rotation_euler[2],
                    -obj.rotation_euler[1]),
                shape_type=shape_type,
                shape_size=shape_size,
                mass=obj[bl.RIGID_WEIGHT],
                linear_damping=obj[bl.RIGID_LINEAR_DAMPING],
                angular_damping=obj[bl.RIGID_ANGULAR_DAMPING],
                restitution=obj[bl.RIGID_RESTITUTION],
                friction=obj[bl.RIGID_FRICTION],
                mode=obj[bl.RIGID_PROCESS_TYPE]
                )
        model.rigidbodies.append(rigidBody)

    # constraint
    model.joints=[pmd.Joint(
        name=obj[bl.CONSTRAINT_NAME].encode('cp932'),
        rigidbody_index_a=rigidNameMap[obj[bl.CONSTRAINT_A]],
        rigidbody_index_b=rigidNameMap[obj[bl.CONSTRAINT_B]],
        position=common.Vector3(
            obj.location[0], 
            obj.location[2], 
            obj.location[1]),
        rotation=common.Vector3(
            -obj.rotation_euler[0], 
            -obj.rotation_euler[2], 
            -obj.rotation_euler[1]),
        translation_limit_min=common.Vector3(
            obj[bl.CONSTRAINT_POS_MIN][0],
            obj[bl.CONSTRAINT_POS_MIN][1],
            obj[bl.CONSTRAINT_POS_MIN][2]
            ),
        translation_limit_max=common.Vector3(
            obj[bl.CONSTRAINT_POS_MAX][0],
            obj[bl.CONSTRAINT_POS_MAX][1],
            obj[bl.CONSTRAINT_POS_MAX][2]
            ),
        rotation_limit_min=common.Vector3(
            obj[bl.CONSTRAINT_ROT_MIN][0],
            obj[bl.CONSTRAINT_ROT_MIN][1],
            obj[bl.CONSTRAINT_ROT_MIN][2]),
        rotation_limit_max=common.Vector3(
            obj[bl.CONSTRAINT_ROT_MAX][0],
            obj[bl.CONSTRAINT_ROT_MAX][1],
            obj[bl.CONSTRAINT_ROT_MAX][2]),
        spring_constant_translation=common.Vector3(
            obj[bl.CONSTRAINT_SPRING_POS][0],
            obj[bl.CONSTRAINT_SPRING_POS][1],
            obj[bl.CONSTRAINT_SPRING_POS][2]),
        spring_constant_rotation=common.Vector3(
            obj[bl.CONSTRAINT_SPRING_ROT][0],
            obj[bl.CONSTRAINT_SPRING_ROT][1],
            obj[bl.CONSTRAINT_SPRING_ROT][2])
        )
        for obj in self.oneSkinMesh.constraints]

    bl.message('write: %s' % path)
    with io.open(path, 'wb') as f:
        return writer.write(f, model)


def _execute(filepath='', **kwargs):
    active=bl.object.getActive()
    if not active:
        print("abort. no active object.")
        return

    ex=exporter.Exporter()
    ex.setup()
    print(ex)

    write(ex, filepath)
    bl.object.activate(active)
    return {'FINISHED'}




#!/usr/bin/env python
# coding: utf-8

import time
import os
import pymeshio.pmx.reader
import opengl.material
import opengl.texture
import opengl.vertexarray


def build(path):
    t=time.time()
    model=pymeshio.pmx.reader.read_from_file(path)
    if not model:
        return
    print(time.time()-t, "sec")
    # build
    basedir=os.path.dirname(path)
    indexedVertexArray=opengl.vertexarray.IndexedVertexArray()
    for v in model.vertices:
        # left-handed y-up to right-handed y-up                
        if v.deform.__class__ is pymeshio.pmx.Bdef1:
            indexedVertexArray.addVertex(
                    (v.position[0], v.position[1], -v.position[2], 1), 
                    (v.normal[0], v.normal[1], -v.normal[2]), 
                    (v.uv[0], v.uv[1]), 
                    (1, 1, 1, 1),
                    v.deform.index0, 0, 1.0)
        elif v.deform.__class__ is pymeshio.pmx.Bdef2:
            indexedVertexArray.addVertex(
                    (v.position[0], v.position[1], -v.position[2], 1), 
                    (v.normal[0], v.normal[1], -v.normal[2]), 
                    (v.uv[0], v.uv[1]), 
                    (1, 1, 1, 1),
                    v.deform.index0, v.deform.index1, v.deform.weight0)
        else:
            print("unknown deform: {0}".format(v.deform))
    
    # material
    textureMap={}
    faceIndex=0
    def indices():
        for i in model.indices:
            yield i
    indexGen=indices()
    for i, m in enumerate(model.materials):
        material=opengl.material.MQOMaterial()
        material.vcol=True
        material.rgba=(
                m.diffuse_color[0], 
                m.diffuse_color[1], 
                m.diffuse_color[2], 
                m.alpha)
        if m.texture_index!=255:
            texturepath=os.path.join(basedir, model.textures[m.texture_index])
            if os.path.isfile(texturepath):
                if not texturepath in textureMap:
                    texture=opengl.texture.Texture(texturepath)
                    textureMap[texturepath]=texture
                material.texture=textureMap[texturepath]
        indices=indexedVertexArray.addMaterial(material)
        indices+=[next(indexGen) for n in range(m.vertex_count)]
    indexedVertexArray.optimize()
    return indexedVertexArray




# coding: utf-8
import bpy
from . import vertexarray
from .. import bl
from ..pymeshio import englishmap


class Morph(object):
    __slots__=['name', 'type', 'offsets']
    def __init__(self, name, type):
        self.name=name
        self.type=type
        self.offsets=[]

    def add(self, index, offset):
        self.offsets.append((index, offset))

    def sort(self):
        self.offsets.sort(key=lambda e: e[0])

    def __str__(self):
        return "<Morph %s>" % self.name


class SSS(object):
    def __init__(self):
        self.use=1


class DefaultMatrial(object):
    def __init__(self):
        self.name='default'
        # diffuse
        self.diffuse_color=[1, 1, 1]
        self.alpha=1
        # specular
        self.specular_toon_size=0
        self.specular_hardness=5
        self.specular_color=[1, 1, 1]
        # ambient
        self.mirror_color=[1, 1, 1]
        # flag
        self.subsurface_scattering=SSS()
        # texture
        self.texture_slots=[]


class OneSkinMesh(object):
    __slots__=['vertexArray', 'morphList', 'rigidbodies', 'constraints', 'armatureObj']
    def __init__(self):
        self.vertexArray=vertexarray.VertexArray()
        self.morphList=[]
        self.rigidbodies=[]
        self.constraints=[]
        self.armatureObj=None

    def __str__(self):
        return "<OneSkinMesh %s, morph:%d>" % (
                self.vertexArray,
                len(self.morphList))

    def build(self, node):
        ############################################################
        # search armature modifier
        ############################################################
        for m in node.o.modifiers:
            if bl.modifier.isType(m, 'ARMATURE'):
                armatureObj=bl.modifier.getArmatureObject(m)
                if not self.armatureObj:
                    self.armatureObj=armatureObj
                elif self.armatureObj!=armatureObj:
                    print("warning! found multiple armature. ignored.", 
                            armatureObj.name)

        if node.o.type.upper()=='MESH':
            self.addMesh(node.o)

        for child in node.children:
            self.build(child)

    def addMesh(self, obj):
        if not bl.object.isVisible(obj):
            return
        self.__mesh(obj)
        self.__skin(obj)
        self.__rigidbody(obj)
        self.__constraint(obj)

    def __getWeightMap(self, obj, mesh):
        # bone weight
        weightMap={}
        secondWeightMap={}
        def setWeight(i, name, w):
            if w>0:
                if i in weightMap:
                    if i in secondWeightMap:
                        # 上位２つのweightを採用する
                        if w<secondWeightMap[i][1]:
                            pass
                        elif w<weightMap[i][1]:
                            # ２つ目を入れ替え
                            secondWeightMap[i]=(name, w)
                        else:
                            # １つ目を入れ替え
                            weightMap[i]=(name, w)
                    else:
                        if w>weightMap[i][1]:
                            # 多い方をweightMapに
                            secondWeightMap[i]=weightMap[i]
                            weightMap[i]=(name, w)
                        else:
                            secondWeightMap[i]=(name, w)
                else:
                    weightMap[i]=(name, w)

        # ToDo bone weightと関係ないvertex groupを除外する
        for i, v in enumerate(mesh.vertices):
            if len(v.groups)>0:
                for g in v.groups:
                    setWeight(i, obj.vertex_groups[g.group].name, g.weight)
            else:
                try:
                    setWeight(i, obj.vertex_groups[0].name, 1)
                except:
                    # no vertex_groups
                    pass

        # 合計値が1になるようにする
        for i in range(len(mesh.vertices)):
            if i in secondWeightMap:
                secondWeightMap[i]=(secondWeightMap[i][0], 1.0-weightMap[i][1])
            elif i in weightMap:
                weightMap[i]=(weightMap[i][0], 1.0)
                secondWeightMap[i]=("", 0)
            else:
                print("no weight vertex")
                weightMap[i]=("", 0)
                secondWeightMap[i]=("", 0)

        return weightMap, secondWeightMap

    def __processFaces(self, obj_name, mesh, weightMap, secondWeightMap):
        default_material=DefaultMatrial()
        # 各面の処理
        for i, face in enumerate(mesh.faces):
            faceVertexCount=bl.face.getVertexCount(face)
            try:
                material=mesh.materials[bl.face.getMaterialIndex(face)]
            except IndexError as e:
                material=default_material
            v=[mesh.vertices[index] for index in bl.face.getVertices(face)]
            uv=bl.mesh.getFaceUV(
                    mesh, i, face, bl.face.getVertexCount(face))
            # flip triangle
            if faceVertexCount==3:
                # triangle
                self.vertexArray.addTriangle(
                        obj_name, material.name,
                        v[2].index, 
                        v[1].index, 
                        v[0].index,
                        v[2].co, 
                        v[1].co, 
                        v[0].co,
                        bl.vertex.getNormal(v[2]), 
                        bl.vertex.getNormal(v[1]), 
                        bl.vertex.getNormal(v[0]),
                        uv[2], 
                        uv[1], 
                        uv[0],
                        weightMap[v[2].index][0],
                        weightMap[v[1].index][0],
                        weightMap[v[0].index][0],
                        secondWeightMap[v[2].index][0],
                        secondWeightMap[v[1].index][0],
                        secondWeightMap[v[0].index][0],
                        weightMap[v[2].index][1],
                        weightMap[v[1].index][1],
                        weightMap[v[0].index][1]
                        )
            elif faceVertexCount==4:
                # quadrangle
                self.vertexArray.addTriangle(
                        obj_name, material.name,
                        v[2].index, 
                        v[1].index, 
                        v[0].index,
                        v[2].co, 
                        v[1].co, 
                        v[0].co,
                        bl.vertex.getNormal(v[2]), 
                        bl.vertex.getNormal(v[1]), 
                        bl.vertex.getNormal(v[0]), 
                        uv[2], 
                        uv[1], 
                        uv[0],
                        weightMap[v[2].index][0],
                        weightMap[v[1].index][0],
                        weightMap[v[0].index][0],
                        secondWeightMap[v[2].index][0],
                        secondWeightMap[v[1].index][0],
                        secondWeightMap[v[0].index][0],
                        weightMap[v[2].index][1],
                        weightMap[v[1].index][1],
                        weightMap[v[0].index][1]
                        )
                self.vertexArray.addTriangle(
                        obj_name, material.name,
                        v[0].index, 
                        v[3].index, 
                        v[2].index,
                        v[0].co, 
                        v[3].co, 
                        v[2].co,
                        bl.vertex.getNormal(v[0]), 
                        bl.vertex.getNormal(v[3]), 
                        bl.vertex.getNormal(v[2]), 
                        uv[0], 
                        uv[3], 
                        uv[2],
                        weightMap[v[0].index][0],
                        weightMap[v[3].index][0],
                        weightMap[v[2].index][0],
                        secondWeightMap[v[0].index][0],
                        secondWeightMap[v[3].index][0],
                        secondWeightMap[v[2].index][0],
                        weightMap[v[0].index][1],
                        weightMap[v[3].index][1],
                        weightMap[v[2].index][1]
                        )

    def __mesh(self, obj):
        if bl.RIGID_SHAPE_TYPE in obj:
            return
        if bl.CONSTRAINT_A in obj:
            return

        bl.message("export: %s" % obj.name)

        # メッシュのコピーを生成してオブジェクトの行列を適用する
        copyMesh, copyObj=bl.object.duplicate(obj)
        copyObj.name="tmp_object"
        if len(copyMesh.vertices)>0:
            # apply transform
            copyMesh.transform(obj.matrix_world)

            # apply modifier
            for m in [m for m in copyObj.modifiers]:
                if m.type=='SOLIDFY':
                    continue
                elif m.type=='ARMATURE':
                    continue
                elif m.type=='MIRROR':
                    bpy.ops.object.modifier_apply(modifier=m.name)
                else:
                    print(m.type)

            weightMap, secondWeightMap=self.__getWeightMap(copyObj, copyMesh)
            self.__processFaces(obj.name, copyMesh, weightMap, secondWeightMap)
        bl.object.delete(copyObj)

    def createEmptyBasicSkin(self):
        self.__getOrCreateMorph('base', 0)

    def __skin(self, obj):
        if not bl.object.hasShapeKey(obj):
            return

        indexRelativeMap={}
        blenderMesh=bl.object.getData(obj)
        baseMorph=None

        # shape keys
        vg=bl.object.getVertexGroup(obj, bl.MMD_SHAPE_GROUP_NAME)

        # base
        used=set()
        for b in bl.object.getShapeKeys(obj):
            if b.name==bl.BASE_SHAPE_NAME:
                baseMorph=self.__getOrCreateMorph('base', 0)
                basis=b

                relativeIndex=0
                for index in vg:
                    v=bl.shapekey.getByIndex(b, index)
                    pos=[v[0], v[1], v[2]]

                    indices=self.vertexArray.getMappedIndex(obj.name, index)
                    for attribute, i in indices.items():
                        if i in used:
                            continue
                        used.add(i)

                        baseMorph.add(i, pos)
                        indexRelativeMap[i]=relativeIndex
                        relativeIndex+=1

                break
        assert(basis)
        #print(basis.name, len(baseMorph.offsets))

        if len(baseMorph.offsets)==0:
            return

        # shape keys
        for b in bl.object.getShapeKeys(obj):
            if b.name==bl.BASE_SHAPE_NAME:
                continue

            #print(b.name)
            morph=self.__getOrCreateMorph(b.name, 4)
            used=set()
            for index, src, dst in zip(
                    range(len(blenderMesh.vertices)),
                    bl.shapekey.get(basis),
                    bl.shapekey.get(b)):
                offset=[dst[0]-src[0], dst[1]-src[1], dst[2]-src[2]]
                if offset[0]==0 and offset[1]==0 and offset[2]==0:
                    continue
                if index in vg:
                    indices=self.vertexArray.getMappedIndex(obj.name, index)
                    for attribute, i in indices.items():
                        if i in used:
                            continue
                        used.add(i) 
                        morph.add(indexRelativeMap[i], offset)
            assert(len(morph.offsets)<len(baseMorph.offsets))

        # sort skinmap
        original=self.morphList[:]
        def getIndex(morph):
            for i, v in enumerate(englishmap.skinMap):
                if v[0]==morph.name:
                    return i
            #print(morph)
            return len(englishmap.skinMap)
        self.morphList.sort(key=getIndex)

    def __rigidbody(self, obj):
        if not bl.RIGID_SHAPE_TYPE in obj:
            return
        self.rigidbodies.append(obj)

    def __constraint(self, obj):
        if not bl.CONSTRAINT_A in obj:
            return
        self.constraints.append(obj)

    def __getOrCreateMorph(self, name, type):
        for m in self.morphList:
            if m.name==name:
                return m
        m=Morph(name, type)
        self.morphList.append(m)
        return m

    def getVertexCount(self):
        return len(self.vertexArray.positions)






#!/usr/bin/env python
# coding: utf-8

import OpenGL.Tk


class Widget(OpenGL.Tk.RawOpengl):
    def __init__(self, master, engine, *args, **kw):
        #super(Widget, self).__init__(master, *args, **kw)
        OpenGL.Tk.RawOpengl.__init__(self, master, *args, **kw)
        self.engine=engine
        self.bind('<Map>', self.onDraw)
        self.bind('<Expose>', self.onDraw)
        self.bind('<Configure>', self.onResize)
        self.bind('<ButtonPress-1>', lambda e: self.engine.onLeftDown(e.x, e.y) and self.onDraw())
        self.bind('<ButtonRelease-1>', lambda e: self.engine.onLeftUp(e.x, e.y) and self.onDraw())
        self.bind('<B1-Motion>', lambda e: self.engine.onMotion(e.x, e.y) and self.onDraw())
        self.bind('<ButtonPress-2>', lambda e: self.engine.onMiddleDown(e.x, e.y) and self.onDraw())
        self.bind('<ButtonRelease-2>', lambda e: self.engine.onMiddleUp(e.x, e.y) and self.onDraw())
        self.bind('<B2-Motion>', lambda e: self.engine.onMotion(e.x, e.y) and self.onDraw())
        self.bind('<ButtonPress-3>', lambda e: self.engine.onRightDown(e.x, e.y) and self.onDraw())
        self.bind('<ButtonRelease-3>', lambda e: self.engine.onRightUp(e.x, e.y) and self.onDraw())
        self.bind('<B3-Motion>', lambda e: self.engine.onMotion(e.x, e.y) and self.onDraw())

    def onDraw(self, *dummy):
        self.tk.call(self._w, 'makecurrent')
        self.update_idletasks()
        self.engine.draw()
        self.tk.call(self._w, 'swapbuffers')

    def onResize(self, event):
        self.engine.onResize(event.width, event.height)
        self.onDraw()




#!/usr/bin/env python
# coding: utf-8

from OpenGL.GL import *
import re
from .baseview import *


DELEGATE_PATTERN=re.compile('^on[A-Z]')

class BaseController(object):
    def __init__(self, view):
        self.isInitialized=False
        self.setView(view)
        self.root=None

    def setRoot(self, root):
        self.root=root
        self.delegate(root)
        self.isInitialized=False

    def setView(self, view):
        self.view=view
        self.delegate(view)

    def delegate(self, to):
        for name in dir(to):  
            if DELEGATE_PATTERN.match(name):
                method = getattr(to, name)  
                setattr(self, name, method)

    def onUpdate(*args):pass
    def onLeftDown(*args):pass
    def onLeftUp(*args):pass
    def onMiddleDown(*args):pass
    def onMiddleUp(*args):pass
    def onRightDown(*args):pass
    def onRightUp(*args):pass
    def onMotion(*args):pass
    def onResize(*args):pass
    def onWheel(*args):pass
    def onKeyDown(*args):pass
    def onInitialize(*args):pass

    def initialize(self):
        self.view.onResize()
        glEnable(GL_DEPTH_TEST)
        # 初期化時の呼び出し
        self.onInitialize()

    def draw(self):
        if not self.isInitialized:
            self.initialize()
            self.isInitialized=True
        # OpenGLバッファのクリア
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        # 投影行列のクリア
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        self.view.updateProjection()
        # モデルビュー行列のクリア
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        # OpenGL描画
        self.view.updateView()
        if self.root:
            self.root.draw()
        glFlush()




# coding: utf-8
###############################################################################
# VPD
###############################################################################
class LineLoader(object):
    """
    行指向の汎用ローダ
    """
    __slots__=['path', 'io', 'end']
    def __str__(self):
        return "<%s current:%d, end:%d>" % (
                self.__class__, self.getPos(), self.getEnd())

    def getPos(self):
        return self.io.tell()

    def getEnd(self):
        return self.end

    def readline(self):
        return (self.io.readline()).strip()

    def isEnd(self):
        return self.io.tell()>=self.end

    def load(self, path, io, end):
        self.path=path
        self.io=io
        self.end=end
        return self.process()

    def process(self):
        """
        dummy. read to end.
        """
        while not self.isEnd():
            self.io.readline()
        return True


class VPDLoader(LineLoader):
    __slots__=['pose']
    def __init__(self):
        super(VPDLoader, self).__init__()
        self.pose=[]

    def __str__(self):
        return "<VPD poses:%d>" % len(self.pose)

    def process(self):
        if self.readline()!="Vocaloid Pose Data file":
            return

        RE_OPEN=re.compile('^(\w+){(.*)')
        RE_OSM=re.compile('^\w+\.osm;')
        RE_COUNT=re.compile('^(\d+);')

        bone_count=-1
        while not self.isEnd():
            line=self.readline()
            if line=='':
                continue
            m=RE_OPEN.match(line)
            if m:
                if not self.parseBone(m.group(2)):
                    raise Exception("invalid bone")
                continue

            m=RE_OSM.match(line)
            if m:
                continue

            m=RE_COUNT.match(line)
            if m:
                bone_count=int(m.group(1))
                continue

        return len(self.pose)==bone_count

    def parseBone(self, name):
        bone=MotionData(name)
        self.pose.append(bone)
        bone.pos=Vector3(*[float(token) for token in self.readline().split(';')[0].split(',')])
        bone.q=Quaternion(*[float(token) for token in self.readline().split(';')[0].split(',')])
        return self.readline()=="}"





# coding: utf-8
"""
common utilities.
"""
import math
import struct
import sys
import io


def unicode(src):
    """
    literal to unicode for python2 and python3 compatiblity.

    in python2 str to unicode.
    in python3 str(as unicode) to str.
    """
    if sys.version_info[0]<3:
        return src.decode('utf-8')
    else:
        return src

"""
common structures.
"""
class Vector2(object):
    """
    2D coordinate for uv value
    """
    __slots__=['x', 'y']
    def __init__(self, x=0, y=0):
        self.x=x
        self.y=y

    def __str__(self):
        return "<%f %f>" % (self.x, self.y)

    def __eq__(self, rhs):
        return self.x==rhs.x and self.y==rhs.y

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def __getitem__(self, key):
        if key==0:
            return self.x
        elif key==1:
            return self.y
        else:
            assert(False)

    def to_tuple(self):
        return (self.x, self.y)

    def cross(self, rhs):
        """cross(outer) product"""
        return self.x*rhs.y-self.y*rhs.x


class Vector3(object):
    """
    3D coordinate for vertex position, normal direction
    """
    __slots__=['x', 'y', 'z']
    def __init__(self, x=0, y=0, z=0):
        self.x=x
        self.y=y
        self.z=z

    def __str__(self):
        return "<%f %.32f %f>" % (self.x, self.y, self.z)

    def __eq__(self, rhs):
        return self.x==rhs.x and self.y==rhs.y and self.z==rhs.z

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def __getitem__(self, key):
        if key==0:
            return self.x
        elif key==1:
            return self.y
        elif key==2:
            return self.z
        else:
            assert(False)

    def to_tuple(self):
        return (self.x, self.y, self.z)

    def __add__(self, r):
        return Vector3(self.x+r.x, self.y+r.y, self.z+r.z)

    def __sub__(self, rhs):
        return Vector3(self.x-rhs.x, self.y-rhs.y, self.z-rhs.z)

    def getSqNorm(self):
        return self.x*self.x + self.y*self.y + self.z*self.z

    def getNorm(self):
        return math.sqrt(self.getSqNorm())

    def normalize(self):
        factor=1.0/self.getNorm()
        self.x*=factor
        self.y*=factor
        self.z*=factor
        return self

    def to_a(self):
        return [self.x, self.y, self.z]

    def dot(self, rhs):
        """dot(inner) product"""
        return self.x*rhs.x + self.y*rhs.y + self.z*rhs.z

    def cross(self, rhs):
        """cross(outer) product"""
        return Vector3(
                self.y*rhs.z - rhs.y*self.z,
                self.z*rhs.x - rhs.z*self.x,
                self.x*rhs.y - rhs.x*self.y,
                )


class Quaternion(object):
    """
    rotation representation in vmd motion
    """
    __slots__=['x', 'y', 'z', 'w']
    def __init__(self, x=0, y=0, z=0, w=1):
        self.x=x
        self.y=y
        self.z=z
        self.w=w

    def __str__(self):
        return "<%f %f %f %f>" % (self.x, self.y, self.z, self.w)

    def __mul__(self, rhs):
        u=numpy.array([self.x, self.y, self.z], 'f')
        v=numpy.array([rhs.x, rhs.y, rhs.z], 'f')
        xyz=self.w*v+rhs.w*u+numpy.cross(u, v)
        q=Quaternion(xyz[0], xyz[1], xyz[2], self.w*rhs.w-numpy.dot(u, v))
        return q

    def dot(self, rhs):
        return self.x*rhs.x+self.y*rhs.y+self.z*rhs.z+self.w*rhs.w

    def getMatrix(self):
        sqX=self.x*self.x
        sqY=self.y*self.y
        sqZ=self.z*self.z
        xy=self.x*self.y
        xz=self.x*self.z
        yz=self.y*self.z
        wx=self.w*self.x
        wy=self.w*self.y
        wz=self.w*self.z
        return numpy.array([
                # 1
                [1-2*sqY-2*sqZ, 2*xy+2*wz, 2*xz-2*wy, 0],
                # 2
                [2*xy-2*wz, 1-2*sqX-2*sqZ, 2*yz+2*wx, 0],
                # 3
                [2*xz+2*wy, 2*yz-2*wx, 1-2*sqX-2*sqY, 0],
                # 4
                [0, 0, 0, 1]],
                'f')

    def getRHMatrix(self):
        x=-self.x
        y=-self.y
        z=self.z
        w=self.w
        sqX=x*x
        sqY=y*y
        sqZ=z*z
        xy=x*y
        xz=x*z
        yz=y*z
        wx=w*x
        wy=w*y
        wz=w*z
        return numpy.array([
                # 1
                [1-2*sqY-2*sqZ, 2*xy+2*wz, 2*xz-2*wy, 0],
                # 2
                [2*xy-2*wz, 1-2*sqX-2*sqZ, 2*yz+2*wx, 0],
                # 3
                [2*xz+2*wy, 2*yz-2*wx, 1-2*sqX-2*sqY, 0],
                # 4
                [0, 0, 0, 1]],
                'f')

    def getRollPitchYaw(self):
        m=self.getMatrix()

        roll = math.atan2(m[0, 1], m[1, 1])
        pitch = math.asin(-m[2, 1])
        yaw = math.atan2(m[2, 0], m[2, 2])

        if math.fabs(math.cos(pitch)) < 1.0e-6:
            roll += m[0, 1] > math.pi if 0.0 else -math.pi
            yaw += m[2, 0] > math.pi if 0.0 else -math.pi

        return roll, pitch, yaw

    def getSqNorm(self):
        return self.x*self.x+self.y*self.y+self.z*self.z+self.w*self.w

    def getNormalized(self):
        f=1.0/self.getSqNorm()
        q=Quaternion(self.x*f, self.y*f, self.z*f, self.w*f)
        return q

    def getRightHanded(self):
        "swap y and z axis"
        return Quaternion(-self.x, -self.z, -self.y, self.w)

    @staticmethod
    def createFromAxisAngle(axis, rad):
        q=Quaternion()
        half_rad=rad/2.0
        c=math.cos(half_rad)
        s=math.sin(half_rad)
        return Quaternion(axis[0]*s, axis[1]*s, axis[2]*s, c)


class RGB(object):
    """
    material color
    """
    __slots__=['r', 'g', 'b']
    def __init__(self, r=0, g=0, b=0):
        self.r=r
        self.g=g
        self.b=b

    def __eq__(self, rhs):
        return self.r==rhs.r and self.g==rhs.g and self.b==rhs.b

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def __getitem__(self, key):
        if key==0:
            return self.r
        elif key==1:
            return self.g
        elif key==2:
            return self.b
        else:
            assert(False)


class RGBA(object):
    """
    material color
    """
    __slots__=['r', 'g', 'b', 'a']
    def __init__(self, r=0, g=0, b=0, a=1):
        self.r=r
        self.g=g
        self.b=b
        self.a=a

    def __eq__(self, rhs):
        return self.r==rhs.r and self.g==rhs.g and self.b==rhs.b and self.a==rhs.a

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def __getitem__(self, key):
        if key==0:
            return self.r
        elif key==1:
            return self.g
        elif key==2:
            return self.b
        elif key==3:
            return self.a
        else:
            assert(False)


"""
utilities
"""
def radian_to_degree(x):
    """darian to deglee"""

    return x/math.pi * 180.0


class ParseException(Exception):
    """
    Exception in reader
    """
    pass


def readall(path):
    """read all bytes from path
    """
    with open(path, "rb") as f:
        return f.read()


class BinaryReader(object):
    """general BinaryReader
    """
    def __init__(self, ios):
        current=ios.tell()
        #ios.seek(0, io.SEEK_END)
        ios.seek(0, 2)
        self.end=ios.tell()
        ios.seek(current)
        self.ios=ios

    def is_end(self):
        #print(self.ios.tell(), self.end)
        return self.ios.tell()>=self.end
        #return not self.ios.readable()

    def unpack(self, fmt, size):
        result=struct.unpack(fmt, self.ios.read(size))
        return result[0]

    def read_int(self, size):
        if size==1:
            return self.unpack("b", size)
        if size==2:
            return self.unpack("h", size)
        if size==4:
            return self.unpack("i", size)
        print("not reach here")
        raise ParseException("invalid int size: "+size)

    def read_uint(self, size):
        if size==1:
            return self.unpack("B", size)
        if size==2:
            return self.unpack("H", size)
        if size==4:
            return self.unpack("I", size)
        print("not reach here")
        raise ParseException("invalid int size: "+size)

    def read_float(self):
        return self.unpack("f", 4)

    def read_vector2(self):
        return Vector2(
                self.read_float(), 
                self.read_float()
                )

    def read_vector3(self):
        return Vector3(
                self.read_float(), 
                self.read_float(), 
                self.read_float()
                )

    def read_rgba(self):
        return RGBA(
                self.read_float(), 
                self.read_float(), 
                self.read_float(),
                self.read_float()
                )

    def read_rgb(self):
        return RGB(
                self.read_float(), 
                self.read_float(), 
                self.read_float()
                )


class WriteException(Exception):
    """
    Exception in writer
    """
    pass


class BinaryWriter(object):
    def __init__(self, ios):
        self.ios=ios

    def write_bytes(self, v, size=None):
        if size:
            self.ios.write(struct.pack("={0}s".format(size), v))
        else:
            self.ios.write(v)

    def write_float(self, v):
        self.ios.write(struct.pack("f", v))

    def write_int(self, v, size):
        if size==1:
            self.ios.write(struct.pack("b", v))
        elif size==2:
            self.ios.write(struct.pack("h", v))
        elif size==4:
            self.ios.write(struct.pack("i", v))
        else:
            raise WriteError("invalid int uint size")

    def write_uint(self, v, size):
        if v==-1:
            if size==1:
                self.ios.write(struct.pack("B", 255))
            elif size==2:
                self.ios.write(struct.pack("H", 65535))
            elif size==4:
                self.ios.write(struct.pack("I", 4294967295))
            else:
                raise WriteError("invalid int uint size")
        else:
            if size==1:
                self.ios.write(struct.pack("B", v))
            elif size==2:
                self.ios.write(struct.pack("H", v))
            elif size==4:
                self.ios.write(struct.pack("I", v))
            else:
                raise WriteError("invalid int uint size")

    def write_vector2(self, v):
        self.ios.write(struct.pack("=2f", v.x, v.y))

    def write_vector3(self, v):
        self.ios.write(struct.pack("=3f", v.x, v.y, v.z))

    def write_rgb(self, v):
        self.ios.write(struct.pack("=3f", v.r, v.g, v.b))

    def write_rgba(self, v):
        self.ios.write(struct.pack("=4f", v.r, v.g, v.b, v.a))





#!/usr/bin/python
# coding: utf-8

from PIL import Image
from OpenGL.GL import *


class Texture(object):

    def __init__(self, path):
        self.path=path
        self.image=None
        self.id=0
        self.isInitialized=False

    def onInitialize(self):
        self.isInitialized=False

    def createTexture(self):
        self.id=glGenTextures(1)
        if self.id==0:
            print("fail to glGenTextures")
            return False
        print("createTexture: %d" % self.id)

        channels=len(self.image.getbands())
        w, h=self.image.size
        glBindTexture(GL_TEXTURE_2D, self.id)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        if channels==4:
            print("RGBA")
            glPixelStorei(GL_UNPACK_ALIGNMENT, 4)
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 
                    0, GL_RGBA, GL_UNSIGNED_BYTE, self.image.tostring())
        elif channels==3:
            print("RGB")
            glPixelStorei(GL_UNPACK_ALIGNMENT, 1)
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, w, h, 
                    0, GL_RGB, GL_UNSIGNED_BYTE, self.image.tostring())

    def begin(self):
        if not self.isInitialized:
            try:
                # load image
                if not self.image:
                    self.image=Image.open(self.path)
                    if self.image:
                        print("load image:", self.path)
                    else:
                        print("failt to load image:", self.path)
                        return
                # createTexture
                if self.image:
                    self.createTexture()
            except Exception as e:
                print(e)
                return
            finally:
                self.isInitialized=True
        if self.id!=0:
            glEnable(GL_TEXTURE_2D)
            glBindTexture(GL_TEXTURE_2D, self.id)

    def end(self):
        glDisable(GL_TEXTURE_2D)




#!BPY
# coding: utf-8

"""
Name: 'Metasequoia (.mqo)...'
Blender: 245
Group: 'Export'
Tooltip: 'Save as Metasequoia MQO File'
"""
__author__= 'ousttrue'
__url__ = ["http://gunload.web.fc2.com/blender/"]
__version__= '2.4'
__bpydoc__ = """\
This script is an exporter to MQO file format.

Usage:

Run this script from "File->Export" menu.

20080128:
20100518: refactoring.
20100606: integrate 2.4 and 2.5.
20100626: refactoring.
20100710: add [apply_modifier] option(2.5 only).
20100714: remove shape_key when apply_modifier. fix material.
20100724: update for Blender2.53.
20101005: update for Blender2.54.
20101228: update for Blender2.55.
20110429: update for Blender2.57b.
"""

bl_addon_info = {
        'category': 'Import/Export',
        'name': 'Export: Metasequioa Model Format (.mqo)',
        'author': 'ousttrue',
        'version': (2, 1),
        'blender': (2, 5, 3),
        'location': 'File > Export',
        'description': 'Export to the Metasequioa Model Format (.mqo)',
        'warning': '', # used for warning icon and text in addons panel
        'wiki_url': 'http://sourceforge.jp/projects/meshio/wiki/FrontPage',
        'tracker_url': 'http://sourceforge.jp/ticket/newticket.php?group_id=5081',
        }

import os
import sys

class MQOMaterial(object):
    __slots__=[
            'name', 'shader', 'r', 'g', 'b', 'a',
            'dif', 'amb', 'emi',
            ]
    def __init__(self, name, shader=3):
        self.name=name
        self.shader=shader
        self.r=0.5
        self.g=0.5
        self.b=0.5
        self.a=1
        self.dif=1
        self.amb=0
        self.emi=0

    def __str__(self):
        return "\"%s\" shader(%d) col(%f %f %f %f) dif(%f) amb(%f) emi(%f)" % (
                self.name, self.shader, self.r, self.g, self.b, self.a,
                self.dif, self.amb, self.emi
                )


# for 2.5
import bpy

# wrapper
from . import bl

def materialToMqo(m):
    material=MQOMaterial(m.name, 3)
    material.r=m.diffuse_color[0]
    material.g=m.diffuse_color[1]
    material.b=m.diffuse_color[2]
    material.a=m.alpha
    material.amb=m.ambient
    material.emi=m.emit
    return material

def apply_transform(vec, matrix):
    x, y, z = vec
    xloc, yloc, zloc = matrix[3][0], matrix[3][1], matrix[3][2]
    return    x*matrix[0][0] + y*matrix[1][0] + z*matrix[2][0] + xloc,\
            x*matrix[0][1] + y*matrix[1][1] + z*matrix[2][1] + yloc,\
            x*matrix[0][2] + y*matrix[1][2] + z*matrix[2][2] + zloc

def convert_to_mqo(vec):
    return vec.x, vec.z, -vec.y


class OutlineNode(object):
    __slots__=['o', 'children']
    def __init__(self, o):
        self.o=o
        self.children=[]

    def __str__(self):
        return "<Node %s>" % self.o


class ObjectInfo(object):
    __slots__=['object', 'depth', 'material_map']
    def __init__(self, o, depth):
        self.object=o
        self.depth=depth
        self.material_map={}

    def __str__(self):
        return "<ObjectInfo %d %s>" % (self.depth, self.object)


class MqoExporter(object):
    __slots__=["materials", "objects", 'scale', 'apply_modifier',]
    def __init__(self, scale, apply_modifier):
        self.objects=[]
        self.materials=[]
        self.scale=scale
        self.apply_modifier=apply_modifier

    def setup(self, scene):
        # 木構造を構築する
        object_node_map={}
        for o in scene.objects:
            object_node_map[o]=OutlineNode(o)
        for node in object_node_map.values():
            if node.o.parent:
                object_node_map[node.o.parent].children.append(node)

        # ルートを得る
        root=object_node_map[scene.objects.active]

        # 情報を集める
        if root.o.type.upper()=='EMPTY':
            # depth調整 
            for node in root.children:
                self.__setup(node)
        else:
            self.__setup(root)

    def __setup(self, node, depth=0):
        info=ObjectInfo(node.o, depth)
        self.objects.append(info)
        if node.o.type.upper()=='MESH':
            # set material index
            for i, m in enumerate(node.o.data.materials):
                info.material_map[i]=self.__getOrAddMaterial(m)
        # recursive
        for child in node.children:
            self.__setup(child, depth+1)
            
    def __getOrAddMaterial(self, material):
        for i, m in enumerate(self.materials):
            if m==material:
                return i
        index=len(self.materials)
        self.materials.append(material)
        return index

    def write(self, path):
        bl.message("open: "+path)
        io=bl.Writer(path, 'cp932')
        self.__write_header(io)
        self.__write_scene(io)
        print("Writing MaterialChunk")
        self.__write_materials(io, os.path.dirname(path))
        print("Writing ObjectChunk")
        for info in self.objects:
            self.__write_object(io, info)
        io.write("Eof\r\n")
        io.flush()
        io.close()

    def __write_header(self, io):
        io.write("Metasequoia Document\r\n")
        io.write("Format Text Ver 1.0\r\n")
        io.write("\r\n")

    def __write_scene(self, io):
        print("Writing SceneChunk")
        io.write("Scene {\r\n")
        io.write("}\r\n")

    def __write_materials(self, io, dirname):
        # each material    
        io.write("Material %d {\r\n" % (len(self.materials)))
        for m in self.materials:
            io.write(str(materialToMqo(m)))
            # ToDo separated alpha texture
            for filename in bl.material.eachTexturePath(m):
                if len(dirname)>0 and filename.startswith(dirname):
                    # 相対パスに変換する
                    filename=filename[len(dirname)+1:]
                io.write(" tex(\"%s\")" % filename)
                break
            io.write("\r\n") 
        # end of chunk
        io.write("}\r\n") 

    def __write_object(self, io, info):
        print(info)

        obj=info.object
        if obj.type.upper()=='MESH' or obj.type.upper()=='EMPTY':
            pass
        else:
            print(obj.type)
            return

        io.write("Object \""+obj.name+"\" {\r\n")

        # depth
        io.write("\tdepth %d\r\n" % info.depth)

        # mirror
        if not self.apply_modifier:
            if bl.modifier.hasType(obj, 'MIRROR'):
                    io.write("\tmirror 1\r\n")
                    io.write("\tmirror_axis 1\r\n")

        if obj.type.upper()=='MESH':
            # duplicate and applyMatrix
            copyMesh, copyObj=bl.object.duplicate(obj)
            # apply transform
            """
            copyObj.scale=obj.scale
            bpy.ops.object.scale_apply()
            copyObj.rotation_euler=obj.rotation_euler
            bpy.ops.object.rotation_apply()
            copyObj.location=obj.location
            bpy.ops.object.location_apply()
            """
            copyMesh.transform(obj.matrix_world)
            # apply modifier
            if self.apply_modifier:
                # remove shape key
                while bl.object.hasShapeKey(copyObj):
                    bpy.ops.object.shape_key_remove()
                for m in [m for m in copyObj.modifiers]:
                    if m.type=='SOLIDFY':
                        continue
                    elif m.type=='ARMATURE':
                        bpy.ops.object.modifier_apply(modifier=m.name)
                    elif m.type=='MIRROR':
                        bpy.ops.object.modifier_apply(modifier=m.name)
                    else:
                        print(m.type)
            # write mesh
            self.__write_mesh(io, copyMesh, info.material_map)
            bl.object.delete(copyObj)

        io.write("}\r\n") # end of object

    def __write_mesh(self, io, mesh, material_map):
        # vertices
        io.write("\tvertex %d {\r\n" % len(mesh.vertices))
        for vert in mesh.vertices:
            x, y, z = convert_to_mqo(vert.co)
            io.write("\t\t%f %f %f\r\n" % 
                    (x*self.scale, y*self.scale, z*self.scale)) # rotate to y-up
        io.write("\t}\r\n")

        # faces
        io.write("\tface %d {\r\n" % len(mesh.faces))
        for i, face in enumerate(mesh.faces):
            count=bl.face.getVertexCount(face)
            # V
            io.write("\t\t%d V(" % count)
            for j in reversed(bl.face.getVertices(face)):
                io.write("%d " % j)
            io.write(")")
            # mat
            if len(mesh.materials):
                io.write(" M(%d)" % 
                        material_map[bl.face.getMaterialIndex(face)])
            # UV
            if bl.mesh.hasUV(mesh) and bl.mesh.hasFaceUV(mesh, i, face):
                io.write(" UV(")
                for uv in reversed(bl.mesh.getFaceUV(mesh, i, face, count)):
                    # reverse vertical value
                    io.write("%f %f " % (uv[0], 1.0-uv[1])) 
                io.write(")")
            io.write("\r\n")
        io.write("\t}\r\n") # end of faces


def _execute(filepath='', scale=10, apply_modifier=False):
    if bl.object.getActive():
        exporter=MqoExporter(scale, apply_modifier)
        exporter.setup(bl.scene.get())
        exporter.write(filepath)
    else:
        bl.message('no active object !')




# coding: utf-8
"""
PMXモデルをインポートする。

1マテリアル、1オブジェクトで作成する。
"""
import os
import bpy
from . import bl
from .pymeshio import pmx
from .pymeshio.pmx import reader


def convert_coord(pos):
    """
    Left handed y-up to Right handed z-up
    """
    return (pos.x, pos.z, pos.y)

def VtoV(v):
    return bl.createVector(v.x, v.y, v.z)

def get_object_name(fmt, index, name):
    """
    object名を作る。最大21バイト
    """
    len_list=[len(name[:i].encode('utf-8')) for i in range(1, len(name)+1, 1)]
    letter_count=0
    prefix=fmt.format(index)
    max_length=21-len(prefix)
    for str_len in len_list:
        if str_len>max_length:
            break
        letter_count+=1
    name=prefix+name[:letter_count]
    print("%s(%d)" % (name, letter_count))
    return name

def __import_joints(joints, rigidbodies):
    print("create joints")
    container=bl.object.createEmpty('Joints')
    layers=[
        True, False, False, False, False, False, False, False, False, False,
        False, False, False, False, False, False, False, False, False, False,
            ]
    material=bl.material.create('joint')
    material.diffuse_color=(1, 0, 0)
    constraintMeshes=[]
    for i, c in enumerate(joints):
        bpy.ops.mesh.primitive_uv_sphere_add(
                segments=8,
                ring_count=4,
                size=0.1,
                location=(c.position.x, c.position.z, c.position.y),
                layers=layers
                )
        meshObject=bl.object.getActive()
        constraintMeshes.append(meshObject)
        mesh=bl.object.getData(meshObject)
        bl.mesh.addMaterial(mesh, material)
        meshObject.name=get_object_name("j{0:02}:", i, c.name)
        #meshObject.draw_transparent=True
        #meshObject.draw_wire=True
        meshObject.draw_type='SOLID'
        rot=c.rotation
        meshObject.rotation_euler=(-rot.x, -rot.z, -rot.y)

        meshObject[bl.CONSTRAINT_NAME]=c.name
        meshObject[bl.CONSTRAINT_A]=rigidbodies[c.rigidbody_index_a].name
        meshObject[bl.CONSTRAINT_B]=rigidbodies[c.rigidbody_index_b].name
        meshObject[bl.CONSTRAINT_POS_MIN]=VtoV(c.translation_limit_min)
        meshObject[bl.CONSTRAINT_POS_MAX]=VtoV(c.translation_limit_max)
        meshObject[bl.CONSTRAINT_ROT_MIN]=VtoV(c.rotation_limit_min)
        meshObject[bl.CONSTRAINT_ROT_MAX]=VtoV(c.rotation_limit_max)
        meshObject[bl.CONSTRAINT_SPRING_POS]=VtoV(c.spring_constant_translation)
        meshObject[bl.CONSTRAINT_SPRING_ROT]=VtoV(c.spring_constant_rotation)

    for meshObject in reversed(constraintMeshes):
        bl.object.makeParent(container, meshObject)

    return container

def __importRigidBodies(rigidbodies, bones):
    print("create rigid bodies")

    container=bl.object.createEmpty('RigidBodies')
    layers=[
        True, False, False, False, False, False, False, False, False, False,
        False, False, False, False, False, False, False, False, False, False,
            ]
    material=bl.material.create('rigidBody')
    rigidMeshes=[]
    for i, rigid in enumerate(rigidbodies):
        if rigid.bone_index==-1:
            # no reference bone
            bone=bones[0]
        else:
            bone=bones[rigid.bone_index]
        pos=rigid.shape_position
        size=rigid.shape_size

        if rigid.shape_type==0:
            bpy.ops.mesh.primitive_ico_sphere_add(
                    location=(pos.x, pos.z, pos.y),
                    layers=layers
                    )
            bpy.ops.transform.resize(
                    value=(size.x, size.x, size.x))
        elif rigid.shape_type==1:
            bpy.ops.mesh.primitive_cube_add(
                    location=(pos.x, pos.z, pos.y),
                    layers=layers
                    )
            bpy.ops.transform.resize(
                    value=(size.x, size.z, size.y))
        elif rigid.shape_type==2:
            bpy.ops.mesh.primitive_cylinder_add(
                    location=(pos.x, pos.z, pos.y),
                    layers=layers
                    )
            bpy.ops.transform.resize(
                    value=(size.x, size.x, size.y))
        else:
            assert(False)

        meshObject=bl.object.getActive()
        mesh=bl.object.getData(meshObject)
        rigidMeshes.append(meshObject)
        bl.mesh.addMaterial(mesh, material)
        meshObject.name=get_object_name("r{0:02}:", i, rigid.name)
        #meshObject.draw_transparent=True
        #meshObject.draw_wire=True
        meshObject.draw_type='WIRE'
        rot=rigid.shape_rotation
        meshObject.rotation_euler=(-rot.x, -rot.z, -rot.y)

        meshObject[bl.RIGID_NAME]=rigid.name
        meshObject[bl.RIGID_SHAPE_TYPE]=rigid.shape_type
        meshObject[bl.RIGID_PROCESS_TYPE]=rigid.mode
        meshObject[bl.RIGID_BONE_NAME]=bone.name
        meshObject[bl.RIGID_GROUP]=rigid.collision_group
        meshObject[bl.RIGID_INTERSECTION_GROUP]=rigid.no_collision_group
        meshObject[bl.RIGID_WEIGHT]=rigid.param.mass
        meshObject[bl.RIGID_LINEAR_DAMPING]=rigid.param.linear_damping
        meshObject[bl.RIGID_ANGULAR_DAMPING]=rigid.param.angular_damping
        meshObject[bl.RIGID_RESTITUTION]=rigid.param.restitution
        meshObject[bl.RIGID_FRICTION]=rigid.param.friction

    for meshObject in reversed(rigidMeshes):
        bl.object.makeParent(container, meshObject)

    return container

def __create_a_material(m, name, textures_and_images):
    """
    materialを作成する

    :Params:
        m
            pymeshio.pmx.Material
        name
            material name
        textures_and_images
            list of (texture, image)
    """
    material = bl.material.create(name)
    # diffuse
    material.diffuse_shader='FRESNEL'
    material.diffuse_color=[m.diffuse_color.r, m.diffuse_color.g, m.diffuse_color.b]
    material.alpha=m.alpha
    # specular
    material.specular_shader='TOON'
    material.specular_color=[m.specular_color.r, m.specular_color.g, m.specular_color.b]
    material.specular_toon_size=int(m.specular_factor)
    # ambient
    material.mirror_color=[m.ambient_color.r, m.ambient_color.g, m.ambient_color.b]
    # todo
    # flag
    # edge_color
    # edge_size
    # other
    material.preview_render_type='FLAT'
    material.use_transparency=True
    # texture
    if m.texture_index!=-1:
        bl.material.addTexture(material, textures_and_images[m.texture_index][0])
    return material

def __create_armature(bones, display_slots):
    """
    armatureを作成する

    :Params:
        bones
            list of pymeshio.pmx.Bone
    """
    armature, armature_object=bl.armature.create()

    # create bones
    bl.armature.makeEditable(armature_object)
    def create_bone(b):
        bone=bl.armature.createBone(armature, b.name)
        # bone position
        bone.head=bl.createVector(*convert_coord(b.position))
        if not b.getConnectionFlag():
            bone.tail=bl.createVector(*convert_coord(b.position))
        elif not b.getVisibleFlag():
            bone.tail=bone.head+bl.createVector(0, 1, 0)

        return bone
    bl_bones=[create_bone(b) for b in bones]

    # build skeleton
    for b, bone in zip(bones, bl_bones):
        assert(b.name==bone.name)
        if b.parent_index!=-1:
            #print("%s -> %s" % (bones[b.parent_index].name, b.name))
            parent_bone=bl_bones[b.parent_index]
            bone.parent=parent_bone
            if b.getConnectionFlag() and b.tail_index!=-1:
                assert(b.tail_index!=0)
                tail_bone=bl_bones[b.tail_index]
                bone.tail=tail_bone.head
                bl.bone.setConnected(tail_bone)
        else:
            print("no parent %s" % b.name)
    bl.armature.update(armature)

    # create ik constraint
    bl.enterObjectMode()
    pose = bl.object.getPose(armature_object)
    for b, bone in zip(bones, bl_bones):
        if b.getIkFlag():
            ik=b.ik
            assert(len(ik.link)<16)
            p_bone=pose.bones[bones[ik.target_index].name]
            assert(p_bone)
            constraint=bl.armature.createIkConstraint(
                    armature_object, p_bone, bone.name,
                    ik.link, ik.limit_radian, ik.loop)
    bl.armature.makeEditable(armature_object)
    bl.armature.update(armature)

    # create bone group
    bl.enterObjectMode()
    pose = bl.object.getPose(armature_object)
    for i, ds in enumerate(display_slots):
        print(ds)
        g=bl.object.createBoneGroup(armature_object, ds.name, "THEME%02d" % (i+1))
        for t, index in ds.references:
            if t==0:
                name=bones[index].name
                try:
                    pose.bones[name].bone_group=g
                except KeyError as e:
                    print("pose %s is not found" % name)

    bl.enterObjectMode()
    return armature_object

def _execute(filepath):
    """
    importerr 本体
    """
    print(filepath)

    model=reader.read_from_file(filepath)
    if not model:
        print("fail to load %s" % filepath)
        return
    print(model)

    # メッシュをまとめるエンプティオブジェクト
    model_name=model.english_name
    if len(model_name)==0:
        model_name=os.path.basename(filepath)
    root_object=bl.object.createEmpty(model_name)
    root_object[bl.MMD_MB_NAME]=model.name
    root_object[bl.MMD_MB_COMMENT]=model.comment
    root_object[bl.MMD_COMMENT]=model.english_comment

    # armatureを作る
    armature_object=__create_armature(model.bones, model.display_slots)
    if armature_object:
        bl.object.makeParent(root_object, armature_object)

    # テクスチャを作る
    texture_dir=os.path.dirname(filepath)
    textures_and_images=[bl.texture.create(os.path.join(texture_dir, t))
            for t in model.textures]
    print(textures_and_images)

    index_generator=(i for i in model.indices)
    # 頂点配列。(Left handed y-up) to (Right handed z-up)
    vertices=[convert_coord(pos)
            for pos in (v.position for v in model.vertices)]

    for i, m in enumerate(model.materials):
        name=get_object_name("{0:02}:", i, m.name)
        ####################
        # material
        ####################
        material=__create_a_material(m, name, textures_and_images)

        ####################
        # mesh object
        ####################
        # object名はutf-8で21byteまで
        mesh, mesh_object=bl.mesh.create(name)
        bl.mesh.addMaterial(mesh, material)
        # activate object
        bl.object.deselectAll()
        bl.object.activate(mesh_object)
        bl.object.makeParent(root_object, mesh_object)

        ####################
        # vertices & faces
        ####################
        indices=[next(index_generator)
                    for _ in range(m.vertex_count)]
        used_indices=set(indices)
        bl.mesh.addGeometry(mesh, vertices,
                [(indices[i], indices[i+1], indices[i+2])
                    for i in range(0, len(indices), 3)])
        assert(len(model.vertices), len(mesh.vertices))

        # assign material
        bl.mesh.addUV(mesh)
        hasTexture=bl.material.hasTexture(material)
        if hasTexture:
            index_gen=(i for i in indices)
            image=(textures_and_images.get[m.texture_index] 
                    if m.texture_index in textures_and_images
                    else None)
        for i, face in enumerate(mesh.faces):
            bl.face.setMaterial(face, 0)
            if hasTexture:
                uv0=model.vertices[next(index_gen)].uv
                uv1=model.vertices[next(index_gen)].uv
                uv2=model.vertices[next(index_gen)].uv
                bl.mesh.setFaceUV(mesh, i, face, [# fix uv
                    (uv0.x, 1.0-uv0.y),
                    (uv1.x, 1.0-uv1.y),
                    (uv2.x, 1.0-uv2.y)
                    ],
                    image)

        ####################
        # armature
        ####################
        if armature_object:
            # armature modifirer
            bl.modifier.addArmature(mesh_object, armature_object)
            # set vertex attributes(normal, bone weights)
            bl.mesh.useVertexUV(mesh)
            for i, (v,  mvert) in enumerate(zip(model.vertices, mesh.vertices)):
                bl.vertex.setNormal(mvert, convert_coord(v.normal))
                if isinstance(v.deform, pmx.Bdef1):
                    bl.object.assignVertexGroup(mesh_object,
                            model.bones[v.deform.index0].name, i, 1.0)
                elif isinstance(v.deform, pmx.Bdef2):
                    bl.object.assignVertexGroup(mesh_object,
                            model.bones[v.deform.index0].name, i, v.deform.weight0)
                    bl.object.assignVertexGroup(mesh_object,
                            model.bones[v.deform.index1].name, i, 1.0-v.deform.weight0)
                else:
                    raise Exception("unknown deform: %s" % v.deform)

        ####################
        # shape keys
        ####################
        # set shape_key pin
        bl.object.pinShape(mesh_object, True)
        # create base key
        baseShapeBlock=bl.object.addShapeKey(mesh_object, bl.BASE_SHAPE_NAME)
        mesh.update()
        for m in model.morphs:
            new_shape_key=bl.object.addShapeKey(mesh_object, m.name)
            for o in m.offsets:
                if isinstance(o, pmx.VertexMorphOffset):
                    bl.shapekey.assign(new_shape_key, 
                            o.vertex_index, 
                            mesh.vertices[o.vertex_index].co+
                            bl.createVector(*convert_coord(o.position_offset)))
                else:
                    raise Exception("unknown morph type: %s" % o)
        # select base shape
        bl.object.setActivateShapeKey(mesh_object, 0)

        #############################
        # clean up not used vertices
        # in the material.
        #############################
        bl.mesh.vertsDelete(mesh, [i for i in range(len(mesh.vertices))
            if i not in used_indices])

    # import rigid bodies
    rigidbody_object=__importRigidBodies(model.rigidbodies, model.bones)
    if rigidbody_object:
        bl.object.makeParent(root_object, rigidbody_object)

    # import joints
    joint_object=__import_joints(model.joints, model.rigidbodies)
    if joint_object:
        bl.object.makeParent(root_object, joint_object)

    return {'FINISHED'}




#!/usr/bin/env python
# coding: utf-8
"""
========================
MikuMikuDance PMX format
========================

file format
~~~~~~~~~~~
* PMDEditor's Lib/PMX仕様/PMX仕様.txt

specs
~~~~~
* textencoding: unicode
* coordinate: left handed y-up(DirectX)
* uv origin: 
* face: only triangle
* backculling: 

"""
__author__="ousttrue"
__license__="zlib"
__versioon__="1.0.0"


import io
import os
import struct
from .. import common



class DifferenceException(Exception):
    pass


class Diff(object):
    def _diff(self, rhs, key):
        l=getattr(self, key)
        r=getattr(rhs, key)
        if l!=r:
            print(l)
            print(r)
            raise DifferenceException(key)

    def _diff_array(self, rhs, key):
        la=getattr(self, key)
        ra=getattr(rhs, key)
        if len(la)!=len(ra):
            raise DifferenceException("%s diffrence %d with %d" % (key, len(la), len(ra)))
        for i, (l, r) in enumerate(zip(la, ra)):
            if isinstance(l, Diff):
                try:
                    l.diff(r)
                except DifferenceException as e:
                    print(i)
                    print(l)
                    print(r)
                    raise DifferenceException("{0}: {1}".format(key, e.message))
            else:
                if l!=r:
                    print(i)
                    print(l)
                    print(r)
                    raise DifferenceException("{0}".format(key))


class Ik(Diff):
    """ik info
    """
    __slots__=[
            'target_index',
            'loop',
            'limit_radian',
            'link',
            ]
    def __init__(self, target_index, loop, limit_radian, link=None):
        self.target_index=target_index
        self.loop=loop
        self.limit_radian=limit_radian
        self.link=link or []

    def __eq__(self, rhs):
        return (
                self.target_index==rhs.target_index
                and self.loop==rhs.loop
                and self.limit_radian==rhs.limit_radian
                and self.link==rhs.link
                )

    def diff(self, rhs):
        self._diff(rhs, 'target_index')
        self._diff(rhs, 'loop')
        self._diff(rhs, 'limit_radian')
        self._diff_array(rhs, 'link')


class IkLink(Diff):
    """ik link info
    """
    __slots__=[
            'bone_index',
            'limit_angle',
            'limit_min',
            'limit_max',
            ]
    def __init__(self, bone_index, limit_angle, limit_min=None, limit_max=None):
        self.bone_index=bone_index
        self.limit_angle=limit_angle
        self.limit_min=limit_min or common.Vector3()
        self.limit_max=limit_max or common.Vector3()

    def __eq__(self, rhs):
        return (
                self.bone_index==rhs.bone_index
                and self.limit_angle==rhs.limit_angle
                and self.limit_min==rhs.limit_min
                and self.limit_max==rhs.limit_max
                )

    def diff(self, rhs):
        self._diff(rhs, 'bone_index')
        self._diff(rhs, 'limit_angle')
        self._diff(rhs, 'limit_min')
        self._diff(rhs, 'limit_max')


class Bone(Diff):
    """material

    Bone: see __init__
    """
    __slots__=[
            'name',
            'english_name',
            'position',
            'parent_index',
            'layer',
            'flag',

            'tail_position',
            'tail_index',
            'effect_index',
            'effect_factor',
            'fixed_axis',
            'local_x_vector',
            'local_z_vector',
            'external_key',
            'ik',
            ]
    def __init__(self,
            name,
            english_name,
            position,
            parent_index,
            layer,
            flag,
            tail_position=None,
            tail_index=-1,
            effect_index=-1,
            effect_factor=0.0,
            fixed_axis=None,
            local_x_vector=None,
            local_z_vector=None,
            external_key=-1,
            ik=None
            ):
        self.name=name
        self.english_name=english_name
        self.position=position
        self.parent_index=parent_index
        self.layer=layer
        self.flag=flag
        self.tail_position=tail_position or common.Vector3()
        self.tail_index=tail_index
        self.effect_index=effect_index
        self.effect_factor=effect_factor
        self.fixed_axis=fixed_axis or common.Vector3()
        self.local_x_vector=local_x_vector or common.Vector3()
        self.local_z_vector=local_z_vector or common.Vector3()
        self.external_key=external_key
        self.ik=ik

    def __eq__(self, rhs):
        return (
                self.name==rhs.name
                and self.english_name==rhs.english_name
                and self.position==rhs.position
                and self.parent_index==rhs.parent_index
                and self.layer==rhs.layer
                and self.flag==rhs.flag
                )

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def diff(self, rhs):
        self._diff(rhs, 'name')
        self._diff(rhs, 'english_name')
        self._diff(rhs, 'position')
        self._diff(rhs, 'parent_index')
        #self._diff(rhs, 'layer')
        self._diff(rhs, 'flag')
        self._diff(rhs, 'tail_position')
        self._diff(rhs, 'tail_index')
        #self._diff(rhs, 'effect_index')
        #self._diff(rhs, 'effect_factor')
        #self._diff(rhs, 'fixed_axis')
        self._diff(rhs, 'local_x_vector')
        self._diff(rhs, 'local_z_vector')
        self._diff(rhs, 'external_key')
        if self.ik and rhs.ik:
            self.ik.diff(rhs.ik)
        else:
            self._diff(rhs, 'ik')

    def getConnectionFlag(self):
        return (self.flag & 0x0001)!=0

    def getVisibleFlag(self):
        return (self.flag & 0x0008)!=0

    def getIkFlag(self):
        return (self.flag & 0x0020)!=0

    def getRotationFlag(self):
        return (self.flag & 0x0100)!=0

    def getTranslationFlag(self):
        return (self.flag & 0x0200)!=0

    def getFixedAxisFlag(self):
        return (self.flag & 0x0400)!=0

    def getLocalCoordinateFlag(self):
        return (self.flag &  0x0800)!=0
    
    def getExternalParentDeformFlag(self):
        return (self.flag &  0x2000)!=0

 
class Material(Diff):
    """material

    Attributes: see __init__
    """
    __slots__=[
            'name',
            'english_name',
            'diffuse_color',
            'alpha',
            'specular_color',
            'specular_factor',
            'ambient_color',
            'flag',
            'edge_color',
            'edge_size',
            'texture_index',
            'sphere_texture_index',
            'sphere_mode',
            'toon_sharing_flag',
            'toon_texture_index',
            'comment',
            'vertex_count',
            ]
    def __init__(self,
            name,
            english_name,
            diffuse_color,
            alpha,
            specular_factor,
            specular_color,
            ambient_color,
            flag,
            edge_color,
            edge_size,
            texture_index,
            sphere_texture_index,
            sphere_mode,
            toon_sharing_flag,
            toon_texture_index=0,
            comment=common.unicode(""),
            vertex_count=0,
            ):
        self.name=name
        self.english_name=english_name
        self.diffuse_color=diffuse_color
        self.alpha=alpha
        self.specular_color=specular_color
        self.specular_factor=specular_factor
        self.ambient_color=ambient_color
        self.flag=flag
        self.edge_color=edge_color
        self.edge_size=edge_size
        self.texture_index=texture_index
        self.sphere_texture_index=sphere_texture_index
        self.sphere_mode=sphere_mode
        self.toon_sharing_flag=toon_sharing_flag
        self.toon_texture_index=toon_texture_index
        self.comment=comment
        self.vertex_count=vertex_count

    def __eq__(self, rhs):
        return (
                self.name==rhs.name
                and self.english_name==rhs.english_name
                and self.diffuse_color==rhs.diffuse_color
                and self.alpha==rhs.alpha
                and self.specular_color==rhs.specular_color
                and self.specular_factor==rhs.specular_factor
                and self.ambient_color==rhs.ambient_color
                and self.flag==rhs.flag
                and self.edge_color==rhs.edge_color
                and self.edge_size==rhs.edge_size
                and self.texture_index==rhs.texture_index
                and self.sphere_texture_index==rhs.sphere_texture_index
                and self.sphere_mode==rhs.sphere_mode
                and self.toon_sharing_flag==rhs.toon_sharing_flag
                and self.toon_texture_index==rhs.toon_texture_index
                and self.comment==rhs.comment
                and self.vertex_count==rhs.vertex_count
                )

    def diff(self, rhs):
        #self._diff(rhs, "name")
        self._diff(rhs, "english_name")
        self._diff(rhs, "diffuse_color")
        self._diff(rhs, "alpha")
        self._diff(rhs, "specular_color")
        self._diff(rhs, "specular_factor")
        self._diff(rhs, "ambient_color")
        self._diff(rhs, "flag")
        self._diff(rhs, "edge_color")
        self._diff(rhs, "edge_size")
        self._diff(rhs, "texture_index")
        self._diff(rhs, "sphere_texture_index")
        self._diff(rhs, "sphere_mode")
        self._diff(rhs, "toon_sharing_flag")
        self._diff(rhs, "toon_texture_index")
        self._diff(rhs, "comment")
        self._diff(rhs, "vertex_count")

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def __str__(self):
        return ("<pmx.Material {name}>".format(
            name=self.english_name
            ))


class Bdef1(Diff):
    """bone deform. use a weight

    Attributes: see __init__
    """
    __slots__=[ 'index0']
    def __init__(self, index0):
        self.index0=index0

    def __str__(self):
        return "<Bdef1 {0}>".format(self.index0)

    def __eq__(self, rhs):
        return self.index0==rhs.index0

    def __ne__(self, rhs):
        return not self.__eq__(rhs)


class Bdef2(Diff):
    """bone deform. use two weights

    Attributes: see __init__
    """
    __slots__=[ 'index0', 'index1', 'weight0']
    def __init__(self, 
            index0,
            index1,
            weight0):
        self.index0=index0
        self.index1=index1
        self.weight0=weight0

    def __str__(self):
        return "<Bdef2 {0}, {1}, {2}>".format(self.index0, self.index1, self.weight0)

    def __eq__(self, rhs):
        return (
                self.index0==rhs.index0
                and self.index1==rhs.index1
                #and self.weight0==rhs.weight0
                and abs(self.weight0-rhs.weight0)<1e-5
                )

    def __ne__(self, rhs):
        return not self.__eq__(rhs)


class Bdef4(Diff):
    """bone deform. use four weights

    Attributes: see __init__
    """
    __slots__=[ 'index0', 'index1', 'index2', 'index3',
            'weight0', 'weight1', 'weight2', 'weight3']
    def __init__(self, 
            index0,
            index1,
            index2,
            index3,
            weight0,
            weight1,
            weight2,
            weight3):
        self.index0=index0
        self.index1=index1
        self.index2=index2
        self.index3=index3
        self.weight0=weight0
        self.weight1=weight1
        self.weight2=weight2
        self.weight3=weight3

    def __str__(self):
        return "<Bdef4 {0}:{1}, {2}:{3}, {4}:{5}, {6}:{7}>".format(
                self.index0, self.index1, self.index2, self.index3,
                self.weight0, self.weight1, self.weight2, self.weight3)

    def __eq__(self, rhs):
        return (
                self.index0==rhs.index0
                and self.index1==rhs.index1
                and self.index2==rhs.index2
                and self.index3==rhs.index3
                and abs(self.weight0-rhs.weight0)<1e-5
                and abs(self.weight1-rhs.weight1)<1e-5
                and abs(self.weight2-rhs.weight2)<1e-5
                and abs(self.weight3-rhs.weight3)<1e-5
                )

    def __ne__(self, rhs):
        return not self.__eq__(rhs)


class Sdef(Diff):
    """bone sphirical deform. use two weights and sphirical params

    Attributes: see __init__
    """
    __slots__=[ 'index0', 'index1', 'weight0', 
            'sdef_c', 'sdef_r0', 'sdef_r1']
    def __init__(self, 
            index0,
            index1,
            weight0,
            sdef_c,
            sdef_r0,
            sdef_r1):
        self.index0=index0
        self.index1=index1
        self.weight0=weight0
        self.sdef_c=sdef_c
        self.sdef_r0=sdef_r0
        self.sdef_r1=sdef_r1

    def __str__(self):
        return "<Sdef {0}, {1}, {2}, {3} {4} {5}>".format(
                self.index0, self.index1, self.weight0, 
                self.sdef_c, self.sdef_r0, self.sdef_r1)

    def __eq__(self, rhs):
        return (
                self.index0==rhs.index0
                and self.index1==rhs.index1
                #and self.weight0==rhs.weight0
                and abs(self.weight0-rhs.weight0)<1e-5
                and self.sdef_c==rhs.sdef_c
                and self.sdef_r0==rhs.sdef_r0
                and self.sdef_r1==rhs.sdef_r1
                )

    def __ne__(self, rhs):
        return not self.__eq__(rhs)


class Vertex(Diff):
    """
    ==========
    pmx vertex
    ==========

    :IVariables:
        position
            Vector3
        normal 
            Vector3
        uv 
            Vector2
        deform
            Bdef1, Bdef2 or Bdef4
        edge_factor
            float
    """
    __slots__=[ 'position', 'normal', 'uv', 'deform', 'edge_factor' ]
    def __init__(self, 
            position, 
            normal, 
            uv, 
            deform, 
            edge_factor):
        self.position=position 
        self.normal=normal
        self.uv=uv
        self.deform=deform
        self.edge_factor=edge_factor

    def __str__(self):
        return "<Vertex position:{0}, normal:{1}, uv:{2}, deform:{3}, edge:{4}".format(
                self.position, self.normal, self.uv, self.deform, self.edge_factor
                )

    def __eq__(self, rhs):
        return (
                self.position==rhs.position
                and self.normal==rhs.normal
                and self.uv==rhs.uv
                and self.deform==rhs.deform
                and self.edge_factor==rhs.edge_factor
                )

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def diff(self, rhs):
        self._diff(rhs, "position")
        self._diff(rhs, "normal")
        self._diff(rhs, "uv")
        self._diff(rhs, "deform")
        self._diff(rhs, "edge_factor")


class Morph(Diff):
    """pmx morph

    Attributes:
        name: 
        english_name: 
        panel:
        morph_type:
        offsets:
    """
    __slots__=[
            'name',
            'english_name',
            'panel',
            'morph_type',
            'offsets',
            ]
    def __init__(self, name, english_name, panel, morph_type, offsets=None):
        self.name=name
        self.english_name=english_name
        self.panel=panel
        self.morph_type=morph_type
        self.offsets=offsets or []

    def __eq__(self, rhs):
        return (
                self.name==rhs.name
                and self.english_name==rhs.english_name
                and self.panel==rhs.panel
                and self.morph_type==rhs.morph_type
                and self.offsets==rhs.offsets
                )

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def diff(self, rhs):
        self._diff(rhs, 'name')
        self._diff(rhs, 'english_name')
        #self._diff(rhs, 'panel')
        self._diff(rhs, 'morph_type')
        self._diff_array(rhs, 'offsets')


class VertexMorphOffset(Diff):
    """pmx vertex morph offset

    Attributes:
        vertex_index:
        position_offset: Vector3
    """
    __slots__=[
            'vertex_index',
            'position_offset',
            ]
    def __init__(self, vertex_index, position_offset):
        self.vertex_index=vertex_index
        self.position_offset=position_offset

    def __eq__(self, rhs):
        return (
                self.vertex_index==rhs.vertex_index 
                and self.position_offset==rhs.position_offset
                )

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def diff(self, rhs):
        self._diff(rhs, 'vertex_index')
        self._diff(rhs, 'position_offset')


class MaterialMorphData(Diff):
    """pmx mateerial morph data

    Attributes:
    """
    __slots__=[
            'material_index',
            'calc_mode',
            'diffuse',
            'specular',
            'specular_factor',
            'ambient',
            'edge_color',
            'edge_size',
            'texture_factor',
            'sphere_texture_factor',
            'toon_texture_factor',
            ]
    def __init__(self, material_index, calc_mode, 
            diffuse, specular, specular_factor,
            ambient, edge_color, edge_size, 
            texture_factor, sphere_texture_factor, toon_texture_factor):
        self.material_index=material_index
        self.calc_mode=calc_mode
        self.diffuse=diffuse
        self.specular=specular
        self.specular_factor=specular_factor
        self.ambient=ambient
        self.edge_color=edge_color
        self.edge_size=edge_size
        self.texture_factor=texture_factor
        self.sphere_texture_factor=sphere_texture_factor
        self.toon_texture_factor=toon_texture_factor


class DisplaySlot(Diff):
    """pmx display slot

    Attributes:
        name: 
        english_name: 
        special_flag:
        references: list of (ref_type, ref_index)
    """
    __slots__=[
            'name',
            'english_name',
            'special_flag',
            'references',
            ]
    def __init__(self, name, english_name, special_flag, references=None):
        self.name=name
        self.english_name=english_name
        self.special_flag=special_flag
        self.references=references or []

    def __str__(self):
        return "<DisplaySlots %s(%d)>" % (self.name, len(self.references))

    def __eq__(self, rhs):
        return (
                self.name==rhs.name
                and self.english_name==rhs.english_name
                and self.special_flag==rhs.special_flag
                and self.references==rhs.references
                )

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def diff(self, rhs):
        self._diff(rhs, 'name')
        self._diff(rhs, 'english_name')
        self._diff(rhs, 'special_flag')
        #self._diff_array(rhs, 'references')


class RigidBodyParam(Diff):
    """pmx rigidbody param(for bullet)

    Attributes:
        mass:
        linear_damping:
        angular_damping:
        restitution:
        friction:
    """
    __slots__=[
            'mass',
            'linear_damping',
            'angular_damping',
            'restitution',
            'friction',
            ]
    def __init__(self, mass, 
            linear_damping, angular_damping, restitution, friction):
        self.mass=mass
        self.linear_damping=linear_damping
        self.angular_damping=angular_damping
        self.restitution=restitution
        self.friction=friction

    def __eq__(self, rhs):
        return (
                self.mass==rhs.mass
                and self.linear_damping==rhs.linear_damping
                and self.angular_damping==rhs.angular_damping
                and self.restitution==rhs.restitution
                and self.friction==rhs.friction
                )

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def diff(self, rhs):
        self._diff(rhs, 'mass')
        self._diff(rhs, 'linear_damping')
        self._diff(rhs, 'angular_damping')
        self._diff_array(rhs, 'restitution')
        self._diff_array(rhs, 'friction')


class RigidBody(Diff):
    """pmx rigidbody

    Attributes:
        name: 
        english_name: 
        bone_index:
        collision_group:
        no_collision_group:
        shape:
        param:
        mode:
    """
    __slots__=[
            'name',
            'english_name',
            'bone_index',
            'collision_group',
            'no_collision_group',
            'shape_type',
            'shape_size',
            'shape_position',
            'shape_rotation',
            'param',
            'mode',
            ]
    def __init__(self,
            name,
            english_name,
            bone_index,
            collision_group,
            no_collision_group,
            shape_type,
            shape_size,
            shape_position,
            shape_rotation,
            mass,
            linear_damping,
            angular_damping,
            restitution,
            friction,
            mode
            ):
        self.name=name
        self.english_name=english_name
        self.bone_index=bone_index
        self.collision_group=collision_group
        self.no_collision_group=no_collision_group
        self.shape_type=shape_type
        self.shape_size=shape_size
        self.shape_position=shape_position
        self.shape_rotation=shape_rotation
        self.param=RigidBodyParam(mass,
                linear_damping, angular_damping,
                restitution, friction)
        self.mode=mode

    def __eq__(self, rhs):
        return (
                self.name==rhs.name
                and self.english_name==rhs.english_name
                and self.bone_index==rhs.bone_index
                and self.collision_group==rhs.collision_group
                and self.no_collision_group==rhs.no_collision_group
                and self.shape_type==rhs.shape_type
                and self.shape_size==rhs.shape_size
                and self.param==rhs.param
                and self.mode==rhs.mode
                )

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def diff(self, rhs):
        self._diff(rhs, 'name')
        self._diff(rhs, 'english_name')
        self._diff(rhs, 'bone_index')
        self._diff(rhs, 'collision_group')
        self._diff(rhs, 'no_collision_group')
        self._diff(rhs, 'shape_type')
        self._diff(rhs, 'shape_size')
        self._diff(rhs, 'shape_position')
        self._diff(rhs, 'shape_rotation')
        self._diff(rhs, 'param')
        self._diff(rhs, 'mode')


class Joint(Diff):
    """pmx joint

    Attributes:
        name: 
        english_name: 
        joint_type:
        rigidbody_index_a:
        rigidbody_index_b:
        position: Vector3
        rotation: Vector3
        translation_limit_min: Vector3
        translation_limit_max: Vector3
        rotation_limit_min: Vector3
        rotation_limit_max: Vector3
        spring_constant_translation: Vector3
        spring_constant_rotation: Vector3
    """
    __slots__=[
            'name',
            'english_name',
            'joint_type',
            'rigidbody_index_a',
            'rigidbody_index_b',
            'position',
            'rotation',
            'translation_limit_min',
            'translation_limit_max',
            'rotation_limit_min',
            'rotation_limit_max',
            'spring_constant_translation',
            'spring_constant_rotation',
            ]
    def __init__(self, name, english_name,
            joint_type,
            rigidbody_index_a,
            rigidbody_index_b,
            position,
            rotation,
            translation_limit_min,
            translation_limit_max,
            rotation_limit_min,
            rotation_limit_max,
            spring_constant_translation,
            spring_constant_rotation
            ):
        self.name=name
        self.english_name=english_name
        self.joint_type=joint_type
        self.rigidbody_index_a=rigidbody_index_a
        self.rigidbody_index_b=rigidbody_index_b
        self.position=position
        self.rotation=rotation
        self.translation_limit_min=translation_limit_min
        self.translation_limit_max=translation_limit_max
        self.rotation_limit_min=rotation_limit_min
        self.rotation_limit_max=rotation_limit_max
        self.spring_constant_translation=spring_constant_translation
        self.spring_constant_rotation=spring_constant_rotation

    def __eq__(self, rhs):
        return (
                self.name==rhs.name
                and self.english_name==rhs.english_name
                and self.joint_type==rhs.joint_type
                and self.rigidbody_index_a==rhs.rigidbody_index_a
                and self.rigidbody_index_b==rhs.rigidbody_index_b
                and self.position==rhs.position
                and self.rotation==rhs.rotation
                and self.translation_limit_min==rhs.translation_limit_min
                and self.translation_limit_max==rhs.translation_limit_max
                and self.rotation_limit_min==rhs.rotation_limit_min
                and self.rotation_limit_max==rhs.rotation_limit_max
                and self.spring_constant_translation==rhs.spring_constant_translation
                and self.spring_constant_rotation==rhs.spring_constant_rotation
                )

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def diff(self, rhs):
        self._diff(rhs, 'name')
        self._diff(rhs, 'joint_type')
        self._diff(rhs, 'rigidbody_index_a')
        self._diff(rhs, 'rigidbody_index_b')
        self._diff(rhs, 'position')
        self._diff(rhs, 'rotation')
        self._diff(rhs, 'translation_limit_min')
        self._diff(rhs, 'translation_limit_max')
        self._diff(rhs, 'rotation_limit_min')
        self._diff(rhs, 'rotation_limit_max')
        self._diff(rhs, 'spring_constant_translation')
        self._diff(rhs, 'spring_constant_rotation')


class Model(Diff):
    """
    ==========
    pmx model
    ==========

    :IVariables:
        version
            pmx version(expected 2.0)
        name 
            model name
        english_name 
            model name
        comment 
            comment
        english_comment 
            comment
        vertices
            vertex list
        textures
            texture list
        materials
            material list
        bones
            bone list
        morph
            morph list
        display_slots
            display list for bone/morph grouping
        rigidbodies
            bullet physics rigidbody list
        joints
            bullet physics joint list
    """
    __slots__=[
            'path',
            'version',
            'name',
            'english_name',
            'comment',
            'english_comment',
            'vertices',
            'indices',
            'textures',
            'materials',
            'bones',
            'morphs',
            'display_slots',
            'rigidbodies',
            'joints',
            ]
    def __init__(self, version=2.0):
        self.path=''
        self.version=version
        self.name=''
        self.english_name=''
        self.comment=''
        self.english_comment=''
        self.vertices=[]
        self.indices=[]
        self.textures=[]
        self.materials=[]
        self.bones=[]
        self.morphs=[]
        self.display_slots=[]
        self.rigidbodies=[]
        self.joints=[]

    def __str__(self):
        return ('<pmx-{version} "{name}" {vertices}vertices>'.format(
            version=self.version,
            name=self.english_name,
            vertices=len(self.vertices)
            ))

    def __eq__(self, rhs):
        return (
                self.version==rhs.version
                and self.name==rhs.name
                and self.english_name==rhs.english_name
                and self.comment==rhs.comment
                and self.english_comment==rhs.english_comment
                and self.vertices==rhs.vertices
                and self.indices==rhs.indices
                and self.textures==rhs.textures
                and self.materials==rhs.materials
                and self.bones==rhs.bones
                and self.morphs==rhs.morphs
                and self.display_slots==rhs.display_slots
                and self.rigidbodies==rhs.rigidbodies
                and self.joints==rhs.joints
                )

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def diff(self, rhs):
        self._diff(rhs, "version")
        self._diff(rhs, "name")
        self._diff(rhs, "english_name")
        self._diff(rhs, "comment")
        self._diff(rhs, "english_comment")
        self._diff_array(rhs, "vertices")
        self._diff_array(rhs, "indices")
        self._diff_array(rhs, "textures")
        self._diff_array(rhs, "materials")
        self._diff_array(rhs, "bones")
        self._diff_array(rhs, "morphs")
        self._diff_array(rhs, "display_slots")
        self._diff_array(rhs, "rigidbodies")
        self._diff_array(rhs, "joints")




# coding: utf-8
"""
vmd reader
"""
import io
import struct
from .. import common
from .. import vmd


class Reader(common.BinaryReader):
    def read_text(self, size):
        """read cp932 text
        """
        src=self.unpack("%ds" % size, size)
        assert(type(src)==bytes)
        pos = src.find(b"\x00")
        if pos==-1:
            return src
        else:
            return src[:pos]

    def read_bone_frame(self):
        """
        フレームひとつ分を読み込む(111 bytes)
        """
        frame=vmd.BoneFrame(self.read_text(15))
        (frame.frame, frame.pos.x, frame.pos.y, frame.pos.z,
        frame.q.x, frame.q.y, frame.q.z, frame.q.w) = struct.unpack(
                'I7f', self.ios.read(32))
        # complement data
        frame.complement=''.join(
                ['%x' % x for x in struct.unpack('64B', self.ios.read(64))])
        return frame

    def read_morph_frame(self):
        """
        モーフデータひとつ分を読み込む(23 bytes)
        """
        frame=vmd.MorphFrame(self.read_text(15))
        (frame.frame, frame.ratio)=struct.unpack('If', self.ios.read(8))
        return frame

    def read_camera_frame(self):
        """
        カメラデータひとつ分を読み込む(61 bytes)
        """
        frame=vmd.CameraFrame()
        (frame.frame, frame.length,
                frame.pos.x, frame.pos.y, frame.pos.z,
                frame.euler.x, frame.euler.y, frame.euler.z
                )=struct.unpack('If3f3f', self.ios.read(32))
        # complement data
        frame.complement=''.join(
                ['%x' % x for x in struct.unpack('24B', self.ios.read(24))])
        (frame.angle, frame.perspective
                )=struct.unpack('fB', self.ios.read(5))
        return frame


def read_from_file(path):
    """
    read from file path

    :Parameters:
      path
        file path

    >>> import pymeshio.vmd.reader
    >>> m=pymeshio.vmd.reader.read_from_file('resources/motion.vmd')
    >>> print(m)

    """
    return read(io.BytesIO(common.readall(path)))


def read(ios):
    assert(isinstance(ios, io.IOBase))
    reader=common.BinaryReader(ios)

    signature=reader.unpack("30s", 30)
    version=None
    if signature[:25] == "Vocaloid Motion Data 0002":
        version=2
    elif signature[:25] == "Vocaloid Motion Data file":
        version=1
    else:
        print("invalid signature", signature)
        return

    reader=Reader(reader.ios)
    motion=vmd.Motion()
    motion.model_name=reader.read_text(20)
    motion.motions=[reader.read_bone_frame() 
            for _ in range(reader.unpack('I', 4))]
    motion.shapes=[reader.read_morph_frame() 
            for _ in range(reader.unpack('I', 4))]
    motion.cameras=[reader.read_camera_frame() 
            for _ in range(reader.unpack('I', 4))]
    motion.lights=[reader.read_light_frame() 
            for _ in range(reader.unpack('I', 4))]
    return motion




#!/usr/bin/python
# coding: utf-8

from OpenGL.GL import *
import numpy

'''
頂点配列

====
属性
====
* 位置
'''
class VertexArray(object):
    def __init__(self, vertices):
        self.vertices=vertices

    def __str__(self):
        return "<VertexArray %d>" % len(self.vertices)

    def draw(self):
        # 位置属性
        glEnableClientState(GL_VERTEX_ARRAY)
        glVertexPointer(3, GL_FLOAT, 0, self.vertices)
        # 描画
        glDrawArrays(GL_TRIANGLES, 0, len(self.vertices))
        # 後始末
        glDisableClientState(GL_VERTEX_ARRAY)



'''
頂点配列

====
属性
====
* 位置
* UV
'''
class VertexArrayWithUV(object):
    def __init__(self, vertices, uvarray):
        self.vertices=vertices
        self.uvarray=uvarray

    def __str__(self):
        return "<VertexArrayWithUV %d>" % len(self.vertices)

    def draw(self):
        # 位置属性
        glEnableClientState(GL_VERTEX_ARRAY)
        glVertexPointer(3, GL_FLOAT, 0, self.vertices)
        # UV属性
        glEnableClientState(GL_TEXTURE_COORD_ARRAY)
        glTexCoordPointer(2, GL_FLOAT, 0, self.uvarray)
        # 描画
        triangle_count=int(len(self.vertices)/3)
        glDrawArrays(GL_TRIANGLES, 0, triangle_count)
        # 後始末
        glDisableClientState(GL_TEXTURE_COORD_ARRAY)
        glDisableClientState(GL_VERTEX_ARRAY)

    def get_boundingbox(self):
        vertices_size=len(self.vertices)
        if(vertices_size==0):
            return ([0, 0, 0], [0, 0, 0])
        def vertex_gen_factory():
            for i in range(0, vertices_size, 3):
                yield [
                        self.vertices[i],
                        self.vertices[i+1],
                        self.vertices[i+2]
                        ]
        vertex_gen=vertex_gen_factory()
        v=next(vertex_gen)
        max_v=v[:]
        min_v=v[:]
        for v in vertex_gen:
            min_v[0]=min(min_v[0], v[0]) 
            min_v[1]=min(min_v[1], v[1]) 
            min_v[2]=min(min_v[2], v[2]) 
            max_v[0]=max(max_v[0], v[0]) 
            max_v[1]=max(max_v[1], v[1]) 
            max_v[2]=max(max_v[2], v[2]) 
        return (min_v, max_v)

'''
インデックス参照頂点配列

====
属性
====
* 位置
* UV
'''
class IndexedVertexArray(object):
    def __init__(self):
        # vertices
        self.vertices=[]
        self.normal=[]
        self.colors=[]
        self.uvlist=[]
        self.b0=[]
        self.b1=[]
        self.w0=[]
        # indices
        self.materials=[]
        self.indicesMap={}

    def addVertex(self, pos, normal, uv, color, b0, b1, w0):
        self.vertices+=pos
        self.normal+=normal
        self.colors+=color
        self.uvlist+=uv
        self.b0.append(b0)
        self.b1.append(b1)
        self.w0.append(w0)

    def addMaterial(self, material):
        self.materials.append(material)
        indices=[]
        self.indicesMap[material]=indices
        return indices

    def draw(self):
        # 位置属性
        glEnableClientState(GL_VERTEX_ARRAY)
        glVertexPointer(4, GL_FLOAT, 0, self.vertices)
        # UV属性
        glEnableClientState(GL_TEXTURE_COORD_ARRAY)
        glTexCoordPointer(2, GL_FLOAT, 0, self.uvlist)
        # マテリアル毎の描画
        for m in self.materials:
            m.begin()
            # 順序維持
            indices=self.indicesMap[m]
            # indexによる描画
            glDrawElements(GL_TRIANGLES, len(indices), GL_UNSIGNED_INT, indices)
            m.end()
        # 後始末
        glDisableClientState(GL_TEXTURE_COORD_ARRAY)
        glDisableClientState(GL_VERTEX_ARRAY)

    def optimize(self):
        self.vertices=numpy.array(self.vertices, numpy.float32) 
        self.uvlist=numpy.array(self.uvlist, numpy.float32) 
        for m, indices in self.indicesMap.items():
            self.indicesMap[m]=numpy.array(indices, numpy.uint32)

    def get_boundingbox(self):
        vertices_size=len(self.vertices)
        if(vertices_size==0):
            return ([0, 0, 0], [0, 0, 0])
        def vertex_gen_factory():
            for i in range(0, vertices_size, 4):
                yield [
                        self.vertices[i],
                        self.vertices[i+1],
                        self.vertices[i+2]
                        ]
        vertex_gen=vertex_gen_factory()
        v=next(vertex_gen)
        max_v=v[:]
        min_v=v[:]
        for v in vertex_gen:
            if v[0]<min_v[0]:
                min_v[0]=v[0]
            if v[1]<min_v[1]:
                min_v[1]=v[1]
            if v[2]<min_v[2]:
                min_v[2]=v[2]
            if v[0]>max_v[0]:
                max_v[0]=v[0]
            if v[1]>max_v[1]:
                max_v[1]=v[1]
            if v[2]>max_v[2]:
                max_v[2]=v[2]
        return (min_v, max_v)




#!/usr/bin/env python
# coding: utf-8

from setuptools import setup, find_packages
import sys
import os
import shutil


# copy pymeshio dir for blender26 plugin
PYMESHIO_DIR_IN_BLENDER26='blender26-meshio/pymeshio'
if os.path.exists(PYMESHIO_DIR_IN_BLENDER26):
    shutil.rmtree(PYMESHIO_DIR_IN_BLENDER26)    
print("copy pymeshio to %s" % PYMESHIO_DIR_IN_BLENDER26)
shutil.copytree('pymeshio', PYMESHIO_DIR_IN_BLENDER26)


setup(
        name='pymeshio',
        version='2.5.1',
        description='3d model io library for mqo, pmd, pmx, vmd and vpd',
        long_description=open('README.rst').read(),
        classifiers=[
            'Programming Language :: Python :: 3',
            'License :: OSI Approved :: zlib/libpng License',
            'Topic :: Multimedia :: Graphics :: 3D Modeling',
            ],
        keywords=['mqo', 'pmd', 'pmx', 'vmd', 'vpd', 'mmd', 'blender'],
        author='ousttrue',
        author_email='ousttrue@gmail.com',
        url='http://pypi.python.org/pypi/pymeshio/',
        license='zlib',
        packages=find_packages(),
        test_suite='nose.collector',
        tests_require=['Nose'],
        zip_safe = (sys.version>="2.5"),   # <2.5 needs unzipped for -m to work
        entry_points = {
            'console_scripts': [
                'pmd2pmx = pymeshio.main:pmd_to_pmx',
                'pmd_diff = pymeshio.main:pmd_diff',
                'pmd_validator = pymeshio.main:pmd_validator',
                ]
            }
        )




#!BPY
# coding:utf-8
"""
 Name: 'MikuMikuDance model (.pmd)...'
 Blender: 248
 Group: 'Import'
 Tooltip: 'Import PMD file for MikuMikuDance.'
"""
__author__= ["ousttrue"]
__url__=()
__bpydoc__="""
pmd Importer

This script imports a pmd into Blender for editing.

20091126: first implement.
20091209: implement IK.
20091210: implement morph target.
20100305: use english name.
20100408: cleanup not used vertices.
20100416: fix fornt face. texture load fail safe. add progress.
20100506: C extension.
20100521: add shape_key group.
20100530: add invisilbe bone tail(armature layer 2).
20100608: integrate 2.4 and 2.5.
20100616: implement rigid body.
20100619: fix for various models.
20100623: fix constraint name.
20100626: refactoring.
20100629: sphere map.
20100703: implement bone group.
20100710: implement toon texture.
20100718: keep model name, comment.
20100724: update for Blender2.53.
20100731: add full python module.
20101005: update for Blender2.54.
20101228: update for Blender2.55.
20110429: update for Blender2.57b.
20110522: implement RigidBody and Constraint.
20110918: update for Blender2.59.
20111002: update for pymeshio-2.1.0
20120306: fix rigid body.
"""
bl_addon_info = {
        'category': 'Import/Export',
        'name': 'Import: MikuMikuDance Model Format (.pmd)',
        'author': 'ousttrue',
        'version': (2, 2),
        'blender': (2, 5, 3),
        'location': 'File > Import',
        'description': 'Import from the MikuMikuDance Model Format (.pmd)',
        'warning': '', # used for warning icon and text in addons panel
        'wiki_url': 'http://sourceforge.jp/projects/meshio/wiki/FrontPage',
        'tracker_url': 'http://sourceforge.jp/ticket/newticket.php?group_id=5081',
        }


###############################################################################
# import
###############################################################################
import os
import sys
import math

from .pymeshio import pmd
from .pymeshio.pmd import reader
from .pymeshio import englishmap

# for 2.5
import bpy
import mathutils

# wrapper
from . import bl

xrange=range

def createPmdMaterial(m, index):
    material = bpy.data.materials.new("Material")
    # diffuse
    material.diffuse_shader='FRESNEL'
    material.diffuse_color=([m.diffuse_color.r, m.diffuse_color.g, m.diffuse_color.b])
    material.alpha=m.alpha
    # specular
    material.specular_shader='TOON'
    material.specular_color=([m.specular_color.r, m.specular_color.g, m.specular_color.b])
    material.specular_toon_size=int(m.specular_factor)
    # ambient
    material.mirror_color=([m.ambient_color.r, m.ambient_color.g, m.ambient_color.b])
    # flag
    material.subsurface_scattering.use=True if m.edge_flag==1 else False
    # other
    material.name="m_%02d" % index
    material.preview_render_type='FLAT'
    material.use_transparency=True
    return material

def poseBoneLimit(n, b):
    if n.endswith("_t"):
        return
    if n.startswith("knee_"):
        b.lock_ik_y=True
        b.lock_ik_z=True
        b.lock_ik_x=False
        # IK limit
        b.use_ik_limit_x=True
        b.ik_min_x=0
        b.ik_max_x=180
    elif n.startswith("ankle_"):
        #b.ik_dof_y=False
        pass

def setSphereMap(material, index, blend_type='MULTIPLY'):
    slot=material.texture_slots[index]
    slot.texture_coords='NORMAL'
    slot.mapping='SPHERE'
    slot.blend_type=blend_type


###############################################################################
def VtoV(v):
    return bl.createVector(v.x, v.y, v.z)


def convert_coord(pos):
    """
    Left handed y-up to Right handed z-up
    """
    return (pos.x, pos.z, pos.y)


def to_radian(degree):
    return math.pi * degree / 180


def get_bone_name(l, index):
    if index==-1:
        return l.bones[0].name.decode('cp932')

    if index < len(l.bones):
        name=englishmap.getEnglishBoneName(l.bones[index].name.decode('cp932'))
        if name:
            return name
        return l.bones[index].name.decode('cp932')
    print('invalid bone index', index)
    return l.bones[0].name.decode('cp932')


def get_group_name(g):
    group_name=englishmap.getEnglishBoneGroupName(g.decode('cp932').strip())
    if not group_name:
        group_name=g.decode('cp932').strip()
    return group_name


def __importToonTextures(io, tex_dir):
    mesh, meshObject=bl.mesh.create(bl.TOON_TEXTURE_OBJECT)
    material=bl.material.create(bl.TOON_TEXTURE_OBJECT)
    bl.mesh.addMaterial(mesh, material)
    for i in range(10):
        toon=io.toon_textures[i]
        path=os.path.join(tex_dir, toon.decode('cp932'))
        texture, image=bl.texture.create(path)
        bl.material.addTexture(material, texture, False)
    return meshObject, material


def __importShape(obj, l, vertex_map):
    if len(l.morphs)==0:
        return

    # set shape_key pin
    bl.object.pinShape(obj, True)

    # find base
    base=None
    for s in l.morphs:
        if s.type==0:
            base=s

            # create vertex group
            bl.object.addVertexGroup(obj, bl.MMD_SHAPE_GROUP_NAME)
            hasShape=False
            for i in s.indices:
                if i in vertex_map:
                    hasShape=True
                    bl.object.assignVertexGroup(
                            obj, bl.MMD_SHAPE_GROUP_NAME, vertex_map[i], 0)
            if not hasShape:
                return
    assert(base)

    # create base key
    baseShapeBlock=bl.object.addShapeKey(obj, bl.BASE_SHAPE_NAME)
    # mesh
    mesh=bl.object.getData(obj)
    mesh.update()

    # each skin
    for s in l.morphs:
        if s.type==0:
            continue

        # name
        name=englishmap.getEnglishSkinName(s.name.decode('cp932'))
        if not name:
            name=s.name.decode('cp932')

        # 25
        new_shape_key=bl.object.addShapeKey(obj, name)

        for index, offset in zip(s.indices, s.pos_list):
            try:
                base_index=base.indices[index]
            except IndexError as msg:
                print(name)
                print(msg)
                print("invalid index %d/%d" % (index, len(base.indices)))
                continue
            try:
                vertex_index=vertex_map[base_index]
                bl.shapekey.assign(new_shape_key, vertex_index,
                        mesh.vertices[vertex_index].co+
                        bl.createVector(*convert_coord(offset)))
            except KeyError as e:
                print('base_index: %d' % base_index)
                print(e)

    # select base shape
    bl.object.setActivateShapeKey(obj, 0)


def __build(armature, b, p, parent):
    name=englishmap.getEnglishBoneName(b.name.decode('cp932'))
    if not name:
        name=b.name.decode('cp932')

    bone=bl.armature.createBone(armature, name)

    if parent and (b.tail_index==0 or b.type==6 or b.type==7 or b.type==9):
        # 先端ボーン
        bone.head = bl.createVector(*convert_coord(b.pos))
        bone.tail=bone.head+bl.createVector(0, 1, 0)
        bone.parent=parent
        if bone.name=="center_t":
            # センターボーンは(0, 1, 0)の方向を向いていないと具合が悪い
            parent.tail=parent.head+bl.createVector(0, 1, 0)
            bone.head=parent.tail
            bone.tail=bone.head+bl.createVector(0, 1, 0)
        else:
            if parent.tail==bone.head:
                pass
            else:
                print('diffurence with parent.tail and head', name)

        if b.type!=9:
            bl.bone.setConnected(bone)
        # armature layer 2
        bl.bone.setLayerMask(bone, [0, 1])
    else:
        # 通常ボーン
        bone.head = bl.createVector(*convert_coord(b.pos))
        bone.tail = bl.createVector(*convert_coord(b.tail))
        if parent:
            bone.parent=parent
            if parent.tail==bone.head:
                bl.bone.setConnected(bone)

    if bone.head==bone.tail:
        bone.tail=bone.head+bl.createVector(0, 1, 0)

    for c in b.children:
        __build(armature, c, b, bone)


def __importArmature(l):
    armature, armature_object=bl.armature.create()

    # build bone
    bl.armature.makeEditable(armature_object)
    for b in l.bones:
        if not b.parent:
            __build(armature, b, None, None)
    bl.armature.update(armature)
    bl.enterObjectMode()

    # IK constraint
    pose = bl.object.getPose(armature_object)
    for ik in l.ik_list:
        target=l.bones[ik.target]
        name = englishmap.getEnglishBoneName(target.name.decode('cp932'))
        if not name:
            name=target.name.decode('cp932')
        p_bone = pose.bones[name]
        if not p_bone:
            print('not found', name)
            continue
        if len(ik.children) >= 16:
            print('over MAX_CHAINLEN', ik, len(ik.children))
            continue
        effector_name=englishmap.getEnglishBoneName(
                l.bones[ik.index].name.decode('cp932'))
        if not effector_name:
            effector_name=l.bones[ik.index].name.decode('cp932')

        constraint=bl.armature.createIkConstraint(armature_object,
                p_bone, effector_name, 
                ik.children, ik.weight, ik.iterations)

    bl.armature.makeEditable(armature_object)
    bl.armature.update(armature)
    bl.enterObjectMode()

    # create bone group
    for i, g in enumerate(l.bone_group_list):
        name=get_group_name(g.name)
        bl.object.createBoneGroup(armature_object, name, "THEME%02d" % (i+1))

    # assign bone to group
    for b_index, g_index in l.bone_display_list:
        # bone
        b=l.bones[b_index]
        bone_name=englishmap.getEnglishBoneName(b.name.decode('cp932'))
        if not bone_name:
            bone_name=b.name.decode('cp932')
        # group
        g=l.bone_group_list[g_index-1]
        group_name=get_group_name(g.name)

        # assign
        pose.bones[bone_name].bone_group=pose.bone_groups[group_name]

    bl.enterObjectMode()

    return armature_object


def __import16MaerialAndMesh(meshObject, l,
        material_order, face_map, tex_dir, toon_material):

    mesh=bl.object.getData(meshObject)
    ############################################################
    # material
    ############################################################
    bl.progress_print('create materials')
    mesh_material_map={}
    textureMap={}
    imageMap={}
    index=0

    for material_index in material_order:
        try:
            m=l.materials[material_index]
            mesh_material_map[material_index]=index
        except KeyError:
            break

        material=createPmdMaterial(m, material_index)

        # main texture
        texture_name=m.texture_file.decode('cp932')
        if texture_name!='':
            for i, t in enumerate(texture_name.split('*')):
                if t in textureMap:
                    texture=textureMap[t]
                else:
                    path=os.path.join(tex_dir, t)
                    texture, image=bl.texture.create(path)
                    textureMap[texture_name]=texture
                    imageMap[material_index]=image
                texture_index=bl.material.addTexture(material, texture)
                if t.endswith('sph'):
                    # sphere map
                    setSphereMap(material, texture_index)
                elif t.endswith('spa'):
                    # sphere map
                    setSphereMap(material, texture_index, 'ADD')

        # toon texture
        toon_index=bl.material.addTexture(
                material,
                bl.material.getTexture(
                    toon_material,
                    0 if m.toon_index==-1 else m.toon_index
                    ),
                False)

        bl.mesh.addMaterial(mesh, material)

        index+=1

    ############################################################
    # vertex
    ############################################################
    bl.progress_print('create vertices')
    # create vertices
    vertices=[]
    for v in l.each_vertex():
        vertices.append(convert_coord(v.pos))

    ############################################################
    # face
    ############################################################
    bl.progress_print('create faces')
    # create faces
    mesh_face_indices=[]
    mesh_face_materials=[]
    used_vertices=set()

    for material_index in material_order:
        face_offset=face_map[material_index]
        m=l.materials[material_index]
        material_faces=l.indices[face_offset:face_offset+m.vertex_count]

        def degenerate(i0, i1, i2):
            """
            縮退しているか？
            """
            return i0==i1 or i1==i2 or i2==i0

        for j in xrange(0, len(material_faces), 3):
            i0=material_faces[j]
            i1=material_faces[j+1]
            i2=material_faces[j+2]
            # flip
            triangle=[i2, i1, i0]
            if degenerate(*triangle):
                continue
            mesh_face_indices.append(triangle[0:3])
            mesh_face_materials.append(material_index)
            used_vertices.add(i0)
            used_vertices.add(i1)
            used_vertices.add(i2)

    ############################################################
    # create vertices & faces
    ############################################################
    bl.mesh.addGeometry(mesh, vertices, mesh_face_indices)

    ############################################################
    # vertex bone weight
    ############################################################
    # create vertex group
    vertex_groups={}
    for v in l.each_vertex():
        vertex_groups[v.bone0]=True
        vertex_groups[v.bone1]=True
    for i in vertex_groups.keys():
        bl.object.addVertexGroup(meshObject, get_bone_name(l, i))

    # vertex params
    bl.mesh.useVertexUV(mesh)
    for i, v, mvert in zip(xrange(len(l.vertices)),
        l.each_vertex(), mesh.vertices):
        # normal, uv
        bl.vertex.setNormal(mvert, convert_coord(v.normal))
        # bone weight
        w1=float(v.weight0)/100.0
        w2=1.0-w1
        bl.object.assignVertexGroup(meshObject, get_bone_name(l, v.bone0),
            i,  w1)
        bl.object.assignVertexGroup(meshObject, get_bone_name(l, v.bone1),
            i,  w2)

    ############################################################
    # face params
    ############################################################
    used_map={}
    bl.mesh.addUV(mesh)
    for i, (face, material_index) in enumerate(
            zip(mesh.faces, mesh_face_materials)):
        try:
            index=mesh_material_map[material_index]
        except KeyError as message:
            print(message, mesh_material_map, m)
            assert(False)
        bl.face.setMaterial(face, index)
        material=mesh.materials[index]
        used_map[index]=True
        if bl.material.hasTexture(material):
            uv_array=[l.getUV(i) for i in bl.face.getIndices(face)]
            bl.mesh.setFaceUV(mesh, i, face,
                    # fix uv
                    [(uv.x, 1.0-uv.y) for uv in uv_array],
                    imageMap.get(index, None))

        # set smooth
        bl.face.setSmooth(face, True)

    mesh.update()

    ############################################################
    # clean up not used vertices
    ############################################################
    bl.progress_print('clean up vertices not used')
    remove_vertices=[]
    vertex_map={}
    for i, v in enumerate(l.each_vertex()):
        if i in used_vertices:
            vertex_map[i]=len(vertex_map)
        else:
            remove_vertices.append(i)

    bl.mesh.vertsDelete(mesh, remove_vertices)

    bl.progress_print('%s created' % mesh.name)
    return vertex_map


def __importMaterialAndMesh(io, tex_dir, toon_material):
    """
    @param l[in] mmd.PMDLoader
    @param filename[in]
    """
    ############################################################
    # shpaeキーで使われるマテリアル優先的に前に並べる
    ############################################################
    # shapeキーで使われる頂点インデックスを集める
    shape_key_used_vertices=set()
    if len(io.morphs)>0:
        # base
        base=None
        for s in io.morphs:
            if s.type!=0:
                continue
            base=s
            break
        assert(base)

        for index in base.indices:
            shape_key_used_vertices.add(index)

    # マテリアルに含まれる頂点がshape_keyに含まれるか否か？
    def isMaterialUsedInShape(offset, m):
        for i in xrange(offset, offset+m.vertex_count):
            if io.indices[i] in shape_key_used_vertices:
                return True

    material_with_shape=set()

    # 各マテリアルの開始頂点インデックスを記録する
    face_map={}
    face_count=0
    for i, m in enumerate(io.materials):
        face_map[i]=face_count
        if isMaterialUsedInShape(face_count, m):
            material_with_shape.add(i)
        face_count+=m.vertex_count

    # shapeキーで使われる頂点のあるマテリアル
    material_with_shape=list(material_with_shape)
    material_with_shape.sort()

    # shapeキーに使われていないマテリアル
    material_without_shape=[]
    for i in range(len(io.materials)):
        if not i in material_with_shape:
            material_without_shape.append(i)

    # メッシュの生成
    def __splitList(l, length):
        for i in range(0, len(l), length):
            yield l[i:i+length]

    def __importMeshAndShape(material16, name):
        mesh, meshObject=bl.mesh.create(name)

        # activate object
        bl.object.deselectAll()
        bl.object.activate(meshObject)

        # shapeキーで使われる順に並べなおしたマテリアル16個分の
        # メッシュを作成する
        vertex_map=__import16MaerialAndMesh(
                meshObject, io, material16, face_map, tex_dir, toon_material)

        # crete shape key
        __importShape(meshObject, io, vertex_map)

        mesh.update()
        return meshObject

    mesh_objects=[__importMeshAndShape(material16, 'with_shape')
        for material16 in __splitList(material_with_shape, 16)]

    mesh_objects+=[__importMeshAndShape(material16, 'mesh')
        for material16 in __splitList(material_without_shape, 16)]

    return mesh_objects


def __importConstraints(io):
    print("create constraint")
    container=bl.object.createEmpty('Constraints')
    layers=[
        True, False, False, False, False, False, False, False, False, False,
        False, False, False, False, False, False, False, False, False, False,
            ]
    material=bl.material.create('constraint')
    material.diffuse_color=(1, 0, 0)
    constraintMeshes=[]
    for i, c in enumerate(io.joints):
        bpy.ops.mesh.primitive_uv_sphere_add(
                segments=8,
                ring_count=4,
                size=0.1,
                location=(c.position.x, c.position.z, c.position.y),
                layers=layers
                )
        meshObject=bl.object.getActive()
        constraintMeshes.append(meshObject)
        mesh=bl.object.getData(meshObject)
        bl.mesh.addMaterial(mesh, material)
        meshObject.name='c_%03d' % i
        #meshObject.draw_transparent=True
        #meshObject.draw_wire=True
        meshObject.draw_type='SOLID'
        rot=c.rotation
        meshObject.rotation_euler=(-rot.x, -rot.z, -rot.y)

        meshObject[bl.CONSTRAINT_NAME]=c.name.decode('cp932')
        meshObject[bl.CONSTRAINT_A]=io.rigidbodies[c.rigidbody_index_a].name.decode('cp932')
        meshObject[bl.CONSTRAINT_B]=io.rigidbodies[c.rigidbody_index_b].name.decode('cp932')
        meshObject[bl.CONSTRAINT_POS_MIN]=VtoV(c.translation_limit_min)
        meshObject[bl.CONSTRAINT_POS_MAX]=VtoV(c.translation_limit_max)
        meshObject[bl.CONSTRAINT_ROT_MIN]=VtoV(c.rotation_limit_min)
        meshObject[bl.CONSTRAINT_ROT_MAX]=VtoV(c.rotation_limit_max)
        meshObject[bl.CONSTRAINT_SPRING_POS]=VtoV(c.spring_constant_translation)
        meshObject[bl.CONSTRAINT_SPRING_ROT]=VtoV(c.spring_constant_rotation)

    for meshObject in reversed(constraintMeshes):
        bl.object.makeParent(container, meshObject)

    return container


def __importRigidBodies(io):
    print("create rigid bodies")

    container=bl.object.createEmpty('RigidBodies')
    layers=[
        True, False, False, False, False, False, False, False, False, False,
        False, False, False, False, False, False, False, False, False, False,
            ]
    material=bl.material.create('rigidBody')
    rigidMeshes=[]
    for i, rigid in enumerate(io.rigidbodies):
        if rigid.bone_index==-1:
            # no reference bone
            bone=io.bones[0]
        else:
            bone=io.bones[rigid.bone_index]
        pos=bone.pos+rigid.shape_position

        # x, y, z -> x, z, y
        if rigid.shape_type==pmd.SHAPE_SPHERE:
            bpy.ops.mesh.primitive_ico_sphere_add(
                    location=(pos.x, pos.z, pos.y),
                    layers=layers
                    )
            w, h, d=rigid.shape_size.to_a()
            bpy.ops.transform.resize(
                    value=[w, w, w])
        elif rigid.shape_type==pmd.SHAPE_BOX:
            bpy.ops.mesh.primitive_cube_add(
                    location=(pos.x, pos.z, pos.y),
                    layers=layers
                    )
            w, h, d=rigid.shape_size.to_a()
            bpy.ops.transform.resize(
                    value=[w, d, h])
        elif rigid.shape_type==pmd.SHAPE_CAPSULE:
            bpy.ops.mesh.primitive_cylinder_add(
                    location=(pos.x, pos.z, pos.y),
                    layers=layers
                    )
            w, h, d=rigid.shape_size.to_a()
            bpy.ops.transform.resize(
                    value=[w, w, h])
        else:
            assert(False)

        meshObject=bl.object.getActive()
        mesh=bl.object.getData(meshObject)
        rigidMeshes.append(meshObject)
        bl.mesh.addMaterial(mesh, material)
        meshObject.name='r_%03d' % i
        meshObject[bl.RIGID_NAME]=rigid.name.decode('cp932')
        #meshObject.draw_transparent=True
        #meshObject.draw_wire=True
        meshObject.draw_type='WIRE'
        rot=rigid.shape_rotation
        meshObject.rotation_euler=(-rot.x, -rot.z, -rot.y)

        # custom properties
        meshObject[bl.RIGID_SHAPE_TYPE]=rigid.shape_type
        meshObject[bl.RIGID_PROCESS_TYPE]=rigid.mode

        bone_name = englishmap.getEnglishBoneName(bone.name.decode('cp932'))
        if not bone_name:
            bone_name=bone.name.decode('cp932')
        meshObject[bl.RIGID_BONE_NAME]=bone_name

        meshObject[bl.RIGID_GROUP]=rigid.collision_group
        meshObject[bl.RIGID_INTERSECTION_GROUP]=rigid.no_collision_group
        meshObject[bl.RIGID_WEIGHT]=rigid.mass
        meshObject[bl.RIGID_LINEAR_DAMPING]=rigid.linear_damping
        meshObject[bl.RIGID_ANGULAR_DAMPING]=rigid.angular_damping
        meshObject[bl.RIGID_RESTITUTION]=rigid.restitution
        meshObject[bl.RIGID_FRICTION]=rigid.friction

    for meshObject in reversed(rigidMeshes):
        bl.object.makeParent(container, meshObject)

    return container


def _execute(filepath=""):
    """
    load pmd file to context.
    """

    # load pmd
    bl.progress_set('load %s' % filepath, 0.0)

    model=reader.read_from_file(filepath)
    if not model:
        bl.message("fail to load %s" % filepath)
        return
    bl.progress_set('loaded', 0.1)

    # create root object
    model_name=model.english_name.decode('cp932')
    if len(model_name)==0:
        model_name=model.name.decode('cp932')
    root=bl.object.createEmpty(model_name)
    root[bl.MMD_MB_NAME]=model.name.decode('cp932')
    root[bl.MMD_MB_COMMENT]=model.comment.decode('cp932')
    root[bl.MMD_COMMENT]=model.english_comment.decode('cp932')

    # toon textures
    tex_dir=os.path.dirname(filepath)
    toonTextures, toonMaterial=__importToonTextures(model, tex_dir)
    bl.object.makeParent(root, toonTextures)

    # import mesh
    mesh_objects=__importMaterialAndMesh(model, tex_dir, toonMaterial)
    for o in mesh_objects:
        bl.object.makeParent(root, o)

    # import armature
    armature_object=__importArmature(model)
    if armature_object:
        bl.object.makeParent(root, armature_object)
        armature = bl.object.getData(armature_object)

        # add armature modifier
        for o in mesh_objects:
            bl.modifier.addArmature(o, armature_object)

        # Limitation
        for n, b in bl.object.getPose(armature_object).bones.items():
            poseBoneLimit(n, b)

    # import rigid bodies
    rigidBodies=__importRigidBodies(model)
    if rigidBodies:
        bl.object.makeParent(root, rigidBodies)

    # import constraints
    constraints=__importConstraints(model)
    if constraints:
        bl.object.makeParent(root, constraints)

    bl.object.activate(root)




#!/usr/bin/env python
# coding: utf-8

from OpenGL.GL import *

class BaseView(object):
    def __init__(self):
        self.x=0
        self.y=0
        self.w=1
        self.h=1
        self.isLeftDown=False
        self.isMiddelDown=False
        self.isRightDown=False

    def updateProjection(self):
        pass

    def updateView(self):
        pass

    def onResize(self, w=None, h=None):
        self.w=w or self.w
        self.h=h or self.h
        glViewport(0, 0, self.w, self.h)

    def onLeftDown(self, x, y):
        self.isLeftDown=True
        self.x=x
        self.y=y

    def onLeftUp(self, x, y):
        self.isLeftDown=False

    def onMiddleDown(self, x, y):
        self.isMiddelDown=True
        self.x=x
        self.y=y

    def onMiddleUp(self, x, y):
        self.isMiddelDown=False

    def onRightDown(self, x, y):
        self.isRightDown=True
        self.x=x
        self.y=y

    def onRightUp(self, x, y):
        self.isRightDown=False

    def onMotion(self, x, y):
        print("onMotion", x, y)

    def printMatrix(self, m):
        print(m[0][0], m[0][1], m[0][2], m[0][3])
        print(m[1][0], m[1][1], m[1][2], m[1][3])
        print(m[2][0], m[2][1], m[2][2], m[2][3])
        print(m[3][0], m[3][1], m[3][2], m[3][3])




# coding: utf-8
"""
convert model
"""

import math
from . import common
from .common import unicode as u
from . import pmx
from . import pmd

class ConvertException(Exception):
    """
    Exception in writer
    """
    pass


def pmd_to_pmx(src):
    """
    return pymeshio.pmx.Model.

    :Parameters:
        src
            pymeshio.pmd.Model
    """
    dst=pmx.Model()
    # model info
    dst.name=src.name.decode("cp932")
    dst.english_name=src.english_name.decode("cp932")
    dst.comment=src.comment.replace(
            b"\n", b"\r\n").decode("cp932")
    dst.english_comment=src.english_comment.replace(
            b"\n", b"\r\n").decode("cp932")
    # vertices
    def createDeform(bone0, bone1, weight0):
        if weight0==0:
            return pmx.Bdef1(bone1)
        elif weight0==100:
            return pmx.Bdef1(bone0)
        else:
            return pmx.Bdef2(bone0, bone1, weight0*0.01)
    dst.vertices=[
            pmx.Vertex(
                v.pos, 
                v.normal, 
                v.uv, 
                createDeform(v.bone0, v.bone1, v.weight0), 
                1.0 if v.edge_flag==0 else 0.0
                )
            for v in src.vertices]
    # indices
    dst.indices=src.indices[:]
    # materials
    texture_map={}
    def get_flag(m):
        return (
                (1 if False  else 0)+
                (2 if (m.edge_flag & 1!=0) else 0)+
                (4 if True else 0)+
                (8 if True else 0)+
                (16 if (m.edge_flag & 1!=0) else 0)
                )
    def get_texture_file(path):
        if len(path)==0:
            return None
        elif path.find(b'*')==-1:
            return path
        else:
            return b'*'.split(path)[0]
    def get_sphere_texture_file(path):
        if len(path)==0:
            return None
        elif path.find(b'*')==-1:
            return None
        else:
            return b'*'.split(path)[1]
    def get_texture_index(path):
        try:
            return texture_map[get_texture_file(path)]
        except KeyError:
            return -1
    def get_sphere_texture_index(path):
        try:
            return texture_map[get_sphere_texture_file(path)]
        except KeyError:
            return -1
    def get_sphere_texture_flag(path):
        sphere_texture=get_sphere_texture_file(path)
        if sphere_texture:
            if sphere_texture.endswith('.sph'):
                return 1
            elif sphere_texture.endswith('.spa'):
                return 2
            else:
                raise ConvertException(
                        "invalid sphere texture: {0}".format(sphere_texture))
        return 0
    def get_toon_shared_flag(m):
        return 1
    def get_toon_index(m):
        return m.toon_index
    for m in src.materials:
        texture=get_texture_file(m.texture_file)
        if texture and not texture in texture_map:
            texture_map[texture]=len(texture_map)
            dst.textures.append(texture.decode("cp932"))
        sphere_texture=get_sphere_texture_file(m.texture_file)
        if sphere_texture and not sphere_texture in texture_map:
            texture_map[sphere_texture]=len(texture_map)
            dst.textures.append(sphere_texture.decode("cp932"))
    dst.materials=[
            pmx.Material(
                name=common.unicode(""),
                english_name=common.unicode(""),
                diffuse_color=m.diffuse_color,
                alpha=m.alpha,
                specular_factor=m.specular_factor,
                specular_color=m.specular_color,
                ambient_color=m.ambient_color,
                flag=get_flag(m),
                edge_color=common.RGBA(0.0, 0.0, 0.0, 1.0),
                edge_size=1.0,
                texture_index=get_texture_index(m.texture_file),
                sphere_texture_index=get_sphere_texture_index(m.texture_file),
                sphere_mode=get_sphere_texture_flag(m.texture_file),
                toon_sharing_flag=get_toon_shared_flag(m),
                toon_texture_index=get_toon_index(m),
                comment=common.unicode(""),
                vertex_count=m.vertex_count
                )
            for i, m in enumerate(src.materials)]
    # bones
    ik_map={}
    for ik in src.ik_list:
        ik_map[ik.index]=ik
    def is_connected(b):
        if isinstance(b, pmd.Bone_Rolling):
            return False
        if isinstance(b, pmd.Bone_Tweak):
            return False
        return True
    def is_rotatable(b):
        if isinstance(b, pmd.Bone_Rotate):
            return True
        if isinstance(b, pmd.Bone_RotateMove):
            return True
        if isinstance(b, pmd.Bone_RotateInfl):
            return True
        if isinstance(b, pmd.Bone_IKRotateInfl):
            return True
        if isinstance(b, pmd.Bone_Rolling):
            return True
        if isinstance(b, pmd.Bone_IKTarget):
            return True
        if isinstance(b, pmd.Bone_IK):
            return True
        if isinstance(b, pmd.Bone_Unvisible):
            return True
        if isinstance(b, pmd.Bone_Tweak):
            return True
    def is_movable(b):
        if isinstance(b, pmd.Bone_RotateMove):
            return True
        if isinstance(b, pmd.Bone_IK):
            return True
    def is_visible(b):
        if isinstance(b, pmd.Bone_Unvisible):
            return False
        if isinstance(b, pmd.Bone_IKTarget):
            return False
        if isinstance(b, pmd.Bone_Tweak):
            return False
        return True
    def is_manupilatable(b):
        return True
    def has_ik(b):
        if isinstance(b, pmd.Bone_IK):
            return True
    def is_external_rotation(b):
        if isinstance(b, pmd.Bone_RotateInfl):
            return True
        if isinstance(b, pmd.Bone_Tweak):
            return True
    def is_fixed_axis(b):
        if isinstance(b, pmd.Bone_Rolling):
            return True
    def is_local_axis(b):
        pass
    def after_physics(b):
        pass
    def external_parent(b):
        pass
    def get_bone_flag(b):
        return (
                (1 if is_connected(b) else 0)+
                (2 if is_rotatable(b) else 0)+
                (4 if is_movable(b) else 0)+
                (8 if is_visible(b) else 0)+

                (16 if is_manupilatable(b) else 0)+
                (32 if has_ik(b) else 0)+
                0+
                0+

                (256 if is_external_rotation(b) else 0)+
                0+
                (1024 if is_fixed_axis(b) else 0)+
                (2048 if is_local_axis(b) else 0)+

                (4096 if after_physics(b) else 0)+
                (8192 if external_parent(b) else 0)
                )
    def get_tail_position(b):
        return common.Vector3()
    def get_tail_index(b):
        if isinstance(b, pmd.Bone_Rolling):
            return -1
        if isinstance(b, pmd.Bone_IKTarget):
            return -1
        if isinstance(b, pmd.Bone_Unvisible):
            return -1
        if isinstance(b, pmd.Bone_Tweak):
            return -1
        return b.tail_index
    def get_ik_link(bone_index):
        b=src.bones[bone_index]
        if b.english_name.find(b'knee')==-1:
            return pmx.IkLink(
                            bone_index, 0, 
                            common.Vector3(), 
                            common.Vector3())
        else:
            return pmx.IkLink(
                            bone_index, 1, 
                            common.Vector3(-3.1415927410125732, 0.0, 0.0), 
                            common.Vector3(-0.00872664619237184524536132812500, 0.0, 0.0))
    def get_ik(b):
        if isinstance(b, pmd.Bone_IK):
            ik=ik_map[b.index]
            return pmx.Ik(
                    ik.target, ik.iterations, ik.weight * 4, [
                        get_ik_link(child) for child in ik.children ])
        return None
    def get_layer(b):
        return 0
    dst.bones=[
            pmx.Bone(
                name=b.name.decode('cp932'),
                english_name=b.english_name.decode('cp932'),
                position=b.pos,
                parent_index=b.parent_index if b.parent_index!=65535 else -1,
                layer=get_layer(b),
                flag=get_bone_flag(b),
                tail_position=get_tail_position(b),
                tail_index=get_tail_index(b),
                effect_index=-1,
                effect_factor=0.0,
                fixed_axis=common.Vector3(),
                local_x_vector=common.Vector3(),
                local_z_vector=common.Vector3(),
                external_key=-1,
                ik=get_ik(b),
                )
            for i, b in enumerate(src.bones)]
    # bones
    def get_panel(m):
        return 1
    if len(src.morphs)>0:
        base=src.morphs[0]
        assert(base.name==b"base")
        dst.morphs=[
                pmx.Morph(
                    name=m.name.decode('cp932'),
                    english_name=m.english_name.decode('cp932'),
                    panel=get_panel(m),
                    morph_type=1,
                    offsets=[pmx.VertexMorphOffset(base.indices[i], pos)
                        for i, pos in zip(m.indices, m.pos_list)]
                    )
                for i, m in enumerate(src.morphs) if m.name!=b"base"]
    # display_slots
    dst.display_slots=[
            pmx.DisplaySlot(u('Root'), u('Root'), 1),
            pmx.DisplaySlot(u('表情'), u('Exp'), 1)]+[
                    pmx.DisplaySlot(
                        name=g.name.strip().decode('cp932'),
                        english_name=g.english_name.strip().decode('cp932'),
                        special_flag=0)
                    for i, g in enumerate(src.bone_group_list)]
    # rigidbodies
    dst.rigidbodies=[
            pmx.RigidBody(
                name=r.name.decode("cp932"),
                english_name=u(""),
                bone_index=r.bone_index,
                collision_group=r.collision_group,
                no_collision_group=r.no_collision_group,
                shape_type=r.shape_type,
                shape_size=r.shape_size,
                shape_position=(r.shape_position+src.bones[0].pos if r.bone_index==-1 
                    else r.shape_position+src.bones[r.bone_index].pos),
                shape_rotation=r.shape_rotation,
                mass=r.mass,
                linear_damping=r.linear_damping,
                angular_damping=r.angular_damping,
                restitution=r.restitution,
                friction=r.friction,
                mode=r.mode
                )
            for i, r in enumerate(src.rigidbodies)]
    # joints
    dst.joints=[
            pmx.Joint(
                j.name.decode('cp932'),
                u(""),
                0,
                j.rigidbody_index_a,
                j.rigidbody_index_b,
                j.position,
                j.rotation,
                j.translation_limit_min,
                j.translation_limit_max,
                j.rotation_limit_min,
                j.rotation_limit_max,
                j.spring_constant_translation,
                j.spring_constant_rotation
                )
            for i, j in enumerate(src.joints)]
    return dst




#!/usr/bin/python
# coding: utf-8

import numpy
from . import vertexarray


'''
頂点配列をマテリアル毎に分別する
'''
class VertexArrayMap(object):
    def __init__(self, materials):
        self.materials=materials
        self.vertexArrayMap={}

    def getVertexArray(self, material_index):
        material=self.materials[material_index]
        if not material in self.vertexArrayMap:
            vertexArray=vertexarray.VertexArray([])
            self.vertexArrayMap[material]=vertexArray
        return self.vertexArrayMap[material]

    def addTriangle(self, material_index, v0, v1, v2):
        vertexArray=self.getVertexArray(material_index)
        vertexArray.vertices.append(v0.x)
        vertexArray.vertices.append(v0.y)
        vertexArray.vertices.append(v0.z)
        vertexArray.vertices.append(v1.x)
        vertexArray.vertices.append(v1.y)
        vertexArray.vertices.append(v1.z)
        vertexArray.vertices.append(v2.x)
        vertexArray.vertices.append(v2.y)
        vertexArray.vertices.append(v2.z)

    def draw(self):
        for m, vertexArray in self.vertexArrayMap.items():
            m.begin()
            vertexArray.draw()
            m.end()

    def optimize(self):
        for v in self.vertexArrayMap.values():
            v.vertices=numpy.array(v.vertices, 'f') 

'''
頂点配列をマテリアル毎に分別する(UV付き)
'''
class VertexArrayMapWithUV(object):
    def __init__(self):
        self.materials=[]
        self.vertexArrayWithUVMap={}

    def addMaterial(self, material):
        self.materials.append(material)
        self.vertexArrayWithUVMap[material]=vertexarray.VertexArrayWithUV([], [])

    def getVertexArray(self, material_index):
        material=self.materials[material_index]
        return self.vertexArrayWithUVMap[material]

    def addTriangle(self, material_index, v0, v1, v2, uv0, uv1, uv2):
        vertexArray=self.getVertexArray(material_index)
        vertexArray.vertices.append(v0.x)
        vertexArray.vertices.append(v0.y)
        vertexArray.vertices.append(v0.z)
        vertexArray.vertices.append(v1.x)
        vertexArray.vertices.append(v1.y)
        vertexArray.vertices.append(v1.z)
        vertexArray.vertices.append(v2.x)
        vertexArray.vertices.append(v2.y)
        vertexArray.vertices.append(v2.z)
        vertexArray.uvarray.append(uv0.x)
        vertexArray.uvarray.append(uv0.y)
        vertexArray.uvarray.append(uv1.x)
        vertexArray.uvarray.append(uv1.y)
        vertexArray.uvarray.append(uv2.x)
        vertexArray.uvarray.append(uv2.y)

    def draw(self):
        for m, vertexArray in self.vertexArrayWithUVMap.items():
            m.begin()
            vertexArray.draw()
            m.end()

    def optimize(self):
        for v in self.vertexArrayWithUVMap.values():
            v.vertices=numpy.array(v.vertices, 'f') 
            v.uvarray=numpy.array(v.uvarray, 'f') 

    def onInitialize(self):
        #[m.onInitialize() for m in self.materials]
        pass

    def get_boundingbox(self):
        if len(self.vertexArrayWithUVMap)==0:
            return ([0, 0, 0], [0, 0, 0])

        gen=iter(self.vertexArrayWithUVMap.values())
        (min_v, max_v)=next(gen).get_boundingbox()
        for va in gen:
            (va_min_v, va_max_v)=va.get_boundingbox()
            min_v[0]=min(min_v[0], va_min_v[0]) 
            min_v[1]=min(min_v[1], va_min_v[1]) 
            min_v[2]=min(min_v[2], va_min_v[2]) 
            max_v[0]=max(max_v[0], va_max_v[0]) 
            max_v[1]=max(max_v[1], va_max_v[1]) 
            max_v[2]=max(max_v[2], va_max_v[2]) 
        return (min_v, max_v)
